/*
	FreeRTOS V8.2.3  Copyright (C) 2016 WITTENSTEIN aerospace & simulation limited.

	This file is a contribution to the FreeRTOS product.

	WITTENSTEIN high integrity systems is a trading name of WITTENSTEIN
	aerospace & simulation ltd, Registered Office: Brown's Court, Long Ashton
	Business Park, Yanley Lane, Long Ashton, Bristol, BS41 9LB, UK.
	Tel: +44 (0) 1275 395 600, fax: +44 (0) 1275 393 630.
	E-mail: info@HighIntegritySystems.com

	http://www.HighIntegritySystems.com
*/

/******************************************************************************
 * NOTE 1:  This project provides two demo applications.  A simple blinky style
 * project, and a more comprehensive test and demo application.  The
 * mainCREATE_SIMPLE_BLINKY_DEMO_ONLY setting in main.c is used to select
 * between the two.  See the notes on using mainCREATE_SIMPLE_BLINKY_DEMO_ONLY
 * in main.c.  This file implements the comprehensive test and demo version.
 *
 * NOTE 2:  This file only contains the source code that is specific to the
 * full demo.  Generic functions, such FreeRTOS hook functions, and functions
 * required to configure the hardware, are defined in main.c.
 ******************************************************************************
 *
 * main_full() creates all the demo application tasks and a software timer, then
 * starts the scheduler.  The web documentation provides more details of the
 * standard demo application tasks, which provide no particular functionality,
 * but do provide a good example of how to use the FreeRTOS API.
 *
 * In addition to the standard demo tasks, the following tasks and tests are
 * defined and/or created within this file:
 *
 * "Reg test" tasks - These fill the core with known values, then check that
 * each register maintains its expected value for the lifetime of the task.
 * Each task uses a different set of values.  The Reg test tasks execute with a
 * very low priority, so get preempted very frequently.
 * A register containing an unexpected value is indicative of an error in the
 * context switching mechanism.
 *
 * "Check" timer - The check software timer period is initially set to three
 * seconds.  The callback function associated with the check software timer
 * checks that all the standard demo tasks, and the register check tasks, are
 * not only still executing, but are executing without reporting any errors.
 * If the check software timer discovers that a task has either stalled, or
 * reported an error, then it changes its own execution period from the initial
 * three seconds, to just 200ms.  The check software timer callback function
 * also toggles the single LED each time it is called.  This provides a visual
 * indication of the system status:  If the LED toggles every three seconds,
 * then no issues have been discovered.  If the LED toggles every 200ms, then
 * an issue has been discovered with at least one task.
 */

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "timers.h"
#include "semphr.h"

/* Standard demo application includes. */
#include "semtest.h"
#include "dynamic.h"
#include "blocktim.h"
#include "countsem.h"
#include "GenQTest.h"
#include "recmutex.h"
#include "death.h"

/* Priorities for the demo application tasks. */
#define mainQUEUE_POLL_PRIORITY				( tskIDLE_PRIORITY + 2UL )
#define mainSEM_TEST_PRIORITY				( tskIDLE_PRIORITY + 1UL )
#define mainBLOCK_Q_PRIORITY				( tskIDLE_PRIORITY + 2UL )
#define mainCREATOR_TASK_PRIORITY			( tskIDLE_PRIORITY + 3UL )
#define mainFLOP_TASK_PRIORITY				( tskIDLE_PRIORITY )

/* A block time of zero simply means "don't block". */
#define mainDONT_BLOCK						( 0UL )

/* The period after which the check timer will expire, in ms, provided no
 * errors have been reported by any of the standard demo tasks.
 * ms are converted to the equivalent in ticks using the portTICK_PERIOD_MS
 * constant. */
#define mainCHECK_TIMER_PERIOD_MS			( 3000UL / portTICK_PERIOD_MS )

/* The period at which the check timer will expire, in ms, if an error has been
 * reported in one of the standard demo tasks.
 * ms are converted to the equivalent in ticks using the portTICK_PERIOD_MS
 * constant. */
#define mainERROR_CHECK_TIMER_PERIOD_MS 	( 200UL / portTICK_PERIOD_MS )

/* Interrupt test constants. */
#define mainINTERRUPT_TEST_CLOCK		( 32768U )
#define mainINTERRUPT_TEST_PRESCALER	( 64U )
#define mainINTERRUPT_TEST_FREQ_HZ		( 4U )
#define mainINTERRUPT_TEST_IRQ_NUM		( 27U )

/* ITIM16 registers. */
#define mainITIM16_ITPRE_2_REG		*( ( volatile uint8_t * )  0xFFF711U )
#define mainITIM16_ITCNT16_2_REG	*( ( volatile uint16_t * ) 0xFFF712U )
#define mainITIM16_ITCTS_2_REG		*( ( volatile uint8_t * )  0xFFF714U )

#define mainITIM16_ITCTS_TO_STS	( 0x01U )
#define mainITIM16_ITCTS_TO_IE	( 0x04U )
#define mainITIM16_ITCTS_TO_WUE	( 0x08U )
#define mainITIM16_ITCTS_CKSEL	( 0x10U )
#define mainITIM16_ITCTS_ITEN	( 0x80U )


/*-----------------------------------------------------------*/

/*
 * The check timer callback function, as described at the top of this file.
 */
static void prvCheckTimerCallback( TimerHandle_t xTimer );

/*
 * Reg test tasks.
 */
extern void vRegTest1Task( void *pvParameters );
extern void vRegTest2Task( void *pvParameters );

/*
 * Interrupt test functions.
 */
static void prvStartInterruptTest( void );
static void prvInterruptTestISR( void );
static void prvInterruptTestTask( void *pvParameter );

/*-----------------------------------------------------------*/

/*
 * Reg test data.
 */
volatile UBaseType_t uxRegTest1Value = 0U;
volatile UBaseType_t uxRegTest2Value = 0U;

/*
 * Interrupt test data.
 */
static SemaphoreHandle_t xInterruptTestSemaphore = NULL;
static UBaseType_t uxInterruptTestValue;

/*-----------------------------------------------------------*/

void main_full( void )
{
TimerHandle_t xCheckTimer = NULL;

	/* Start all the other standard demo/test tasks.  The have not particular
	 * functionality, but do demonstrate how to use the FreeRTOS API and test
	 * the kernel port. */
	vStartDynamicPriorityTasks();
	vCreateBlockTimeTasks();
	vStartCountingSemaphoreTasks();
	vStartGenericQueueTasks( tskIDLE_PRIORITY );
	vStartRecursiveMutexTasks();
	vStartSemaphoreTasks( mainSEM_TEST_PRIORITY );

	/* Start Reg test. */
	xTaskCreate( vRegTest1Task, "Reg1", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY, NULL );
	xTaskCreate( vRegTest2Task, "Reg2", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY, NULL );

	/* Start Interrupt test. */
	prvStartInterruptTest();

	/* Create the software timer that performs the 'check' functionality,
	 * as described at the top of this file. */
	xCheckTimer = xTimerCreate( "CheckTimer",				/* A text name, purely to help debugging. */
								mainCHECK_TIMER_PERIOD_MS,	/* The timer period, in this case 3000ms (3s). */
								pdTRUE,						/* This is an auto-reload timer, so xAutoReload is set to pdTRUE. */
								NULL,						/* The ID is not used, so can be set to anything. */
								prvCheckTimerCallback		/* The callback function that inspects the status of all the other tasks. */
							  );

	if( xCheckTimer != NULL )
	{
		xTimerStart( xCheckTimer, mainDONT_BLOCK );
	}

	/* This task has to be created last as it keeps account of the number of
	 * tasks it expects to see running. */
	vCreateSuicidalTasks( mainCREATOR_TASK_PRIORITY );

	/* Start the scheduler. */
	vTaskStartScheduler();

	/* If all is well, the scheduler will now be running, and the following
	 * line will never be reached.  If the following line does execute, then
	 * there was insufficient FreeRTOS heap memory available for the idle
	 * and/or timer tasks to be created.  See the memory management section on
	 * the FreeRTOS web site for more details. */
	for( ;; );
}
/*-----------------------------------------------------------*/

static void prvCheckTimerCallback( TimerHandle_t xTimer )
{
static BaseType_t xChangedTimerPeriodAlready = pdFALSE;
BaseType_t xErrorFound = pdFALSE;

static UBaseType_t uxLastRegTest1Value = 0U;
static UBaseType_t uxLastRegTest2Value = 0U;

	/* Check all the demo tasks (other than the flash tasks) to ensure that
	 * they are all still running, and that none have detected an error. */

	if( xAreDynamicPriorityTasksStillRunning() != pdTRUE )
	{
		xErrorFound = pdTRUE;
	}

	if ( xAreBlockTimeTestTasksStillRunning() != pdTRUE )
	{
		xErrorFound = pdTRUE;
	}

	if ( xAreCountingSemaphoreTasksStillRunning() != pdTRUE )
	{
		xErrorFound = pdTRUE;
	}

	if ( xAreGenericQueueTasksStillRunning() != pdTRUE )
	{
		xErrorFound = pdTRUE;
	}

	if ( xAreRecursiveMutexTasksStillRunning() != pdTRUE )
	{
		xErrorFound = pdTRUE;
	}

	if( xAreSemaphoreTasksStillRunning() != pdTRUE )
	{
		xErrorFound = pdTRUE;
	}

	if( uxLastRegTest1Value == uxRegTest1Value )
	{
		xErrorFound = pdTRUE;
	}
	uxLastRegTest1Value = uxRegTest1Value;

	if( uxLastRegTest2Value == uxRegTest2Value )
	{
		xErrorFound = pdTRUE;
	}
	uxLastRegTest2Value = uxRegTest2Value;

	/* Toggle the check LED to give an indication of the system status.
	 * If the LED toggles every mainCHECK_TIMER_PERIOD_MS milliseconds then
	 * everything is ok.  A faster toggle indicates an error. */
	configTOGGLE_LED();

	/* Have any errors been latch in xErrorFound?  If so, shorten the period
	 * of the check timer to mainERROR_CHECK_TIMER_PERIOD_MS milliseconds.
	 * This will result in an increase in the rate at which mainCHECK_LED
	 * toggles. */
	if( xErrorFound != pdFALSE )
	{
		if( xChangedTimerPeriodAlready == pdFALSE )
		{
			xChangedTimerPeriodAlready = pdTRUE;

			/* This call to xTimerChangePeriod() uses a zero block time.
			 * Functions called from inside of a timer callback function must
			 * *never* attempt to block. */
			( void ) xTimerChangePeriod( xTimer, mainERROR_CHECK_TIMER_PERIOD_MS, mainDONT_BLOCK );
		}
	}
}
/*-----------------------------------------------------------*/

static void prvStartInterruptTest( void )
{
UBaseType_t uxCounterValue;
volatile UBaseType_t uxDelay;

	/* Disable timeout interrupt. */
	mainITIM16_ITCTS_2_REG &= ~mainITIM16_ITCTS_TO_IE;

	/* Stop ITIM16 module. */
	do
	{
		mainITIM16_ITCTS_2_REG &= ~mainITIM16_ITCTS_ITEN;

		for( uxDelay = 0; uxDelay < 10U; uxDelay++ ); /* delay. */
	}
	while( 0U != ( mainITIM16_ITCTS_2_REG & mainITIM16_ITCTS_ITEN ) );

    /* Set 32KHz clock. */
	mainITIM16_ITCTS_2_REG |= mainITIM16_ITCTS_CKSEL;

	/* Enable timeout wake-up. */
	mainITIM16_ITCTS_2_REG |= mainITIM16_ITCTS_TO_WUE;

	/* Set prescaler. */
	mainITIM16_ITPRE_2_REG = ( uint8_t ) ( mainINTERRUPT_TEST_PRESCALER - 1 );

	/* Set counter. */
	uxCounterValue = ( ( mainINTERRUPT_TEST_CLOCK / mainINTERRUPT_TEST_FREQ_HZ ) / mainINTERRUPT_TEST_PRESCALER ) - 1U;
	mainITIM16_ITCNT16_2_REG = ( uint16_t ) uxCounterValue;

	/* Disable interrupt. */
	ICU_EnableInt( mainINTERRUPT_TEST_IRQ_NUM, FALSE );

	/* Register ISR. */
	axFreeRTOSIsrSwDispatchTable[ mainINTERRUPT_TEST_IRQ_NUM ] = prvInterruptTestISR;

	/* Clear interrupt. */
	ICU_ClearInt( mainINTERRUPT_TEST_IRQ_NUM );

	/* Enable interrupt. */
	ICU_EnableInt( mainINTERRUPT_TEST_IRQ_NUM, TRUE );

	/* Enable ITIM16 interrupt. */
	mainITIM16_ITCTS_2_REG |= mainITIM16_ITCTS_TO_IE;

	/* Enable module. */
	mainITIM16_ITCTS_2_REG |= mainITIM16_ITCTS_ITEN;

	/* Create the task */
	xTaskCreate( prvInterruptTestTask, "Intr", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY, NULL );

	/* Create the semaphore that is used to demonstrate a task being
	 * synchronised with an interrupt. */
	vSemaphoreCreateBinary( xInterruptTestSemaphore );
}
/*-----------------------------------------------------------*/

static void prvInterruptTestISR( void )
{
BaseType_t xHigherPriorityTaskWoken = pdFALSE;

	/* Acknowledge */
	mainITIM16_ITCTS_2_REG |= mainITIM16_ITCTS_TO_STS;

	/* This interrupt does nothing more than demonstrate how to synchronise a
	 * task with an interrupt.  First the handler releases a semaphore.
	 * xHigherPriorityTaskWoken has been initialised to zero. */
	xSemaphoreGiveFromISR( xInterruptTestSemaphore, &xHigherPriorityTaskWoken );

	/* If there was a task that was blocked on the semaphore, and giving the
	 * semaphore caused the task to unblock, and the unblocked task has a
	 * priority higher than the currently executing task (the task that this
	 * interrupt interrupted), then lHigherPriorityTaskWoken will have been set
	 * to pdTRUE.
	 * Passing pdTRUE into the following macro call will cause this interrupt
	 * to return directly to the unblocked, higher priority, task. */
	portEND_SWITCHING_ISR( xHigherPriorityTaskWoken );
}
/*-----------------------------------------------------------*/


static void prvInterruptTestTask( void *pvParameter )
{
	configASSERT( xInterruptTestSemaphore );

	/* This is the task used as an example of how to synchronise a task with
	 * an interrupt.  Each time the button interrupt gives the semaphore, this
	 * task will unblock, increment its execution counter, then return to block
	 * again. */

	/* Take the semaphore before started to ensure it is in the correct state. */
	xSemaphoreTake( xInterruptTestSemaphore, mainDONT_BLOCK );

	for( ;; )
	{
		xSemaphoreTake( xInterruptTestSemaphore, portMAX_DELAY );
		uxInterruptTestValue++;
	}
}
/*-----------------------------------------------------------*/
