/*
	FreeRTOS V8.2.3  Copyright (C) 2016 WITTENSTEIN aerospace & simulation limited.

	This file is a contribution to the FreeRTOS product.

	WITTENSTEIN high integrity systems is a trading name of WITTENSTEIN
	aerospace & simulation ltd, Registered Office: Brown's Court, Long Ashton
	Business Park, Yanley Lane, Long Ashton, Bristol, BS41 9LB, UK.
	Tel: +44 (0) 1275 395 600, fax: +44 (0) 1275 393 630.
	E-mail: info@HighIntegritySystems.com

	http://www.HighIntegritySystems.com
*/

/*-----------------------------------------------------------------------------
 * Implementation of functions defined in portable.h for the CR16C port.
 *---------------------------------------------------------------------------*/

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"

/*-----------------------------------------------------------*/

/* Tick configuration. */
#define portTICK_PRESCALER		( 10U )
#define portTICK_INTR_NUM		( 28U )

/* Constants for stack initialisation. */
#define portINITIAL_PSR			( 0x0A00U )

/* ICU registers. */
#define portICU_IVCT_REG		*( ( volatile uint16_t * ) 0xFFFE00U )
#define portICU_IENAM0_REG		*( ( volatile uint16_t * ) 0xFFFE0EU )
#define portICU_IENAM1_REG		*( ( volatile uint16_t * ) 0xFFFE10U )
#define portICU_IECLR0_REG		*( ( volatile uint16_t * ) 0xFFFE12U )
#define portICU_IECLR1_REG		*( ( volatile uint16_t * ) 0xFFFE14U )

/* ITIM16 registers. */
#define portITIM16_ITPRE_1_REG		*( ( volatile uint8_t * )  0xFFF701U )
#define portITIM16_ITCNT16_1_REG	*( ( volatile uint16_t * ) 0xFFF702U )
#define portITIM16_ITCTS_1_REG		*( ( volatile uint8_t * )  0xFFF704U )

#define portITIM16_ITCTS_TO_STS	( 0x01U )
#define portITIM16_ITCTS_TO_IE	( 0x04U )
#define portITIM16_ITCTS_TO_WUE	( 0x08U )
#define portITIM16_ITCTS_CKSEL	( 0x10U )
#define portITIM16_ITCTS_ITEN	( 0x80U )

/*-----------------------------------------------------------*/

volatile BaseType_t xPortContextSwitch = pdFALSE;

PortISR_t axFreeRTOSIsrSwDispatchTable[ portICU_INTERRUPT_NUM ] = { 0 };

/*-----------------------------------------------------------*/

/* 
 * Setup the timer to generate the tick interrupts.
 */
static void prvSetupTimerInterrupt( void );

/*
 * Call back for the alarm function.
 */
static void prvPortSysTickHandler( void );

/*-----------------------------------------------------------*/

/* 
 * See header file for description. 
 */
StackType_t *pxPortInitialiseStack( StackType_t *pxTopOfStack, TaskFunction_t pxCode, void *pvParameters )
{    
	/* Initialise registers. */
	*pxTopOfStack-- = 0x000A;					/* RA_H */
	*pxTopOfStack-- = 0x0A00;					/* RA_L */
	*pxTopOfStack-- = 0x0013;					/* R13_H */
	*pxTopOfStack-- = 0x1300;					/* R13_L */
	*pxTopOfStack-- = 0x0012;					/* R12_H */
	*pxTopOfStack-- = 0x1200;					/* R12_L */
	*pxTopOfStack-- = 0x1111;					/* R11 */
	*pxTopOfStack-- = 0x1010;					/* R10 */
	*pxTopOfStack-- = 0x0909;					/* R9 */
	*pxTopOfStack-- = 0x0808;					/* R8 */
	*pxTopOfStack-- = 0x0707;					/* R7 */
	*pxTopOfStack-- = 0x0606;					/* R6 */
	*pxTopOfStack-- = 0x0505;					/* R5 */
	*pxTopOfStack-- = 0x0404;					/* R4 */
	*pxTopOfStack-- = ( StackType_t )( ( ( uint32_t ) pvParameters ) >> 16 );	/* R3 */
	*pxTopOfStack-- = ( StackType_t )( ( ( uint32_t ) pvParameters ) );			/* R2 */
	*pxTopOfStack-- = 0x0101;					/* R1 */
	*pxTopOfStack-- = 0x0000;					/* R0 */

	/* Processor Status. */
	*pxTopOfStack-- = portINITIAL_PSR;

	/* Return address. */
	*pxTopOfStack-- = ( StackType_t )( ( ( uint32_t ) pxCode ) >> 16 );
	*pxTopOfStack = ( StackType_t )( ( ( uint32_t ) pxCode ) );

	return pxTopOfStack;
}
/*-----------------------------------------------------------*/

/* 
 * See header file for description. 
 */
BaseType_t xPortStartScheduler( void )
{
	/* Start the timer that generates the tick ISR. Interrupts are disabled
	 * here already. */
	prvSetupTimerInterrupt();
	
	/* Start the first task. */
	__asm__ volatile
	(
		/* Restore stack pointer. */
		"	loadd	_pxCurrentTCB, (r1,r0)		\n"
		"	loadd	0(r1,r0), (sp)				\n"

		/* Restore return address. */
		"	pop		$2, r0						\n"
		"	stord	(r1,r0), 34(sp)				\n"

		/* Restore processor status. */
		"	pop		$1, r0						\n"
		"	lpr		r0, psr						\n"

		/* Restore scratch registers. */
		"	pop		$8, r0						\n"

		/* Restore safe registers and return address, and return. */
		"	popret	$8, r8, ra					\n"
	);

	/* Should not get here! */
	return 0;
}
/*-----------------------------------------------------------*/

void vPortEndScheduler( void )
{
	/* It is unlikely that this port will require this function as there is
	 * nothing to return to.  */
}
/*-----------------------------------------------------------*/

/*
 * Setup the ITIM16 timer to generate the tick interrupts at the required
 * frequency.
 */
static void prvSetupTimerInterrupt( void )
{
UBaseType_t uxCounterValue;
volatile UBaseType_t uxDelay;

	uxCounterValue = ( ( configCPU_CLOCK_HZ / configTICK_RATE_HZ ) / portTICK_PRESCALER ) - 1U;

    /* Disable timeout interrupt. */
	portITIM16_ITCTS_1_REG &= ~portITIM16_ITCTS_TO_IE;

	/* Stop ITIM16 module. */
	do
	{
		portITIM16_ITCTS_1_REG &= ~( portITIM16_ITCTS_ITEN | portITIM16_ITCTS_CKSEL );

		for( uxDelay = 0; uxDelay < 10U; uxDelay++ ); /* delay. */
	}
	while( 0U != ( portITIM16_ITCTS_1_REG & ( portITIM16_ITCTS_ITEN | portITIM16_ITCTS_CKSEL ) ) );

    /* Enable timeout wake-up. */
	portITIM16_ITCTS_1_REG |= portITIM16_ITCTS_TO_WUE;

	/* Set prescaler. */
	portITIM16_ITPRE_1_REG = ( uint8_t ) ( portTICK_PRESCALER - 1 );

	/* Set counter. */
	portITIM16_ITCNT16_1_REG = ( uint16_t ) uxCounterValue;

	/* Disable interrupt. */
	portICU_IENAM1_REG &= ~( 1U << ( portTICK_INTR_NUM - 16U ) );

	/* Register ISR. */
	axFreeRTOSIsrSwDispatchTable[ portTICK_INTR_NUM ] = prvPortSysTickHandler;

	/* Clear interrupt. */
	portICU_IECLR1_REG = 1U << ( portTICK_INTR_NUM - 16U );

	/* Enable interrupt. */
	portICU_IENAM1_REG |= 1U << ( portTICK_INTR_NUM - 16U );

	/* Enable ITIM16 interrupt. */
	portITIM16_ITCTS_1_REG |= portITIM16_ITCTS_TO_IE;

	/* Enable module. */
	portITIM16_ITCTS_1_REG |= portITIM16_ITCTS_ITEN;
}
/*-----------------------------------------------------------*/

static void prvPortSysTickHandler( void )
{
	/* Acknowledge */
	portITIM16_ITCTS_1_REG |= portITIM16_ITCTS_TO_STS;

	/* Increment the kernel tick. */
	if( xTaskIncrementTick() != pdFALSE )
	{
		xPortContextSwitch = pdTRUE;
	}
}
/*-----------------------------------------------------------*/

void vPortIsrDispatcher( void )
{
UBaseType_t uxVector;

	/* Retrieve vector number. */
	uxVector = portICU_IVCT_REG - portICU_TRAP_NUM;

	configASSERT( uxVector < portICU_INTERRUPT_NUM );

	/* Clear the interrupt source. */
	if ( uxVector < 16U )
	{
		portICU_IECLR0_REG = 1U << uxVector;
	}
	else
	{
		portICU_IECLR1_REG = 1U << ( uxVector - 16U );
	}

	/* Dispatch. */
	axFreeRTOSIsrSwDispatchTable[ uxVector ]();
}
/*-----------------------------------------------------------*/
