/*---------------------------------------------------------------------------------------------------------*/
/*  Nuvoton Technology Corporation confidential                                                            */
/*                                                                                                         */
/*  Copyright (c) 2009 by Nuvoton Technology Corporation                                                   */
/*  All rights reserved                                                                                    */
/*                                                                                                         */
/*<<<------------------------------------------------------------------------------------------------------*/
/* File Contents:                                                                                          */
/*   npce9mnx.c                                                                                            */
/*            This file contains chip dependent functionality implementation                               */
/* Project:                                                                                                */
/*            SWC_HAL                                                                                      */
/*---------------------------------------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                                 INCLUDES                                                */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
#include "npce9mnx_if.h"
#include "npce9mnx_regs.h"

#if defined (SMB_MODULE_TYPE)
static UINT8 SMB_Enabled;
static ICU_INT_SRC_T SmbFifoICU;
#endif

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                           INTERFACE FUNCTIONS                                           */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_SMB_INT                                                                           */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  smb_module - SMB Module number                                                         */
/*                                                                                                         */
/* Returns:         The resperctive Interrupt number of the given SMB module.                              */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine retrieves the interrupt number of the corresponding SMB module.           */
/*---------------------------------------------------------------------------------------------------------*/
#if ( defined SMB_MODULE_TYPE) && ( defined ICU_MODULE_TYPE)
ICU_INT_SRC_T CHIP_SMB_INT(SMB_MODULE_T module)
{
    switch (module)
    {
        case SMB_MODULE_1:
            return ICU_INT_13;
        case SMB_MODULE_2:
            return ICU_INT_14;
        case SMB_MODULE_3:
            return ICU_INT_4;
        case SMB_MODULE_4:
            return ICU_INT_5;
        case SMB_MODULE_5:
            return SmbFifoICU;
        default:
            return ICU_INT_UNDEF;
    }
}
#endif

#if defined (SMB_MODULE_TYPE)
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_SMB_LowVoltageConfig                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  smb_module - SMB Module number                                                         */
/*                  enable  -    TRUE - Enable, FALSE - Disable                                            */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine enable (or disables) the selected SMBus low voltage (1.8V) support.       */
/*---------------------------------------------------------------------------------------------------------*/
void CHIP_SMB_LowVoltageConfig(SMB_MODULE_T smb_module, BOOLEAN enable)
{
    UINT8 const mask[4] = {MASK_FIELD(LV_GPIO_CTL3_SD1_LV) | MASK_FIELD(LV_GPIO_CTL3_SC1_LV),
                           MASK_FIELD(LV_GPIO_CTL3_SD2_LV) | MASK_FIELD(LV_GPIO_CTL3_SC2_LV),
                           MASK_FIELD(LV_GPIO_CTL3_SD3_LV) | MASK_FIELD(LV_GPIO_CTL3_SC3_LV),
                           MASK_FIELD(LV_GPIO_CTL3_SD4_LV) | MASK_FIELD(LV_GPIO_CTL3_SC4_LV)};
        
    if (enable) {
        SET_REG_MASK(LV_GPIO_CTL3, mask[smb_module]);
    }
    else {
        CLEAR_REG_MASK(LV_GPIO_CTL3, mask[smb_module]);
    }
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_SMB_Configure                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  Smb1_enable  -    Enable SMB1                                                          */
/*                  Smb2_enable  -    Enable SMB2                                                          */
/*                  Smb3A_enable -    Enable SMB3A                                                         */
/*                  Smb3B_enable -    Enable SMB3B                                                         */
/*                  Smb4A_enable -    Enable SMB4A                                                         */
/*                  Smb4B_enable -    Enable SMB4B                                                         */
/*                  SmbF_mux     -    SMBF MUX select                                                      */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine selects the SMBx function to the corresponding pin.                       */
/*---------------------------------------------------------------------------------------------------------*/
BOOLEAN CHIP_SMB_Configure (
    BOOLEAN Smb1_enable,
    BOOLEAN Smb2_enable,
    BOOLEAN Smb3A_enable,
    BOOLEAN Smb3B_enable,
    BOOLEAN Smb4A_enable,
    BOOLEAN Smb4B_enable,
    SMBF_MUX_T SmbF_mux
)
{
    SMB_Enabled = 0;
    SmbFifoICU = ICU_INT_UNDEF;
    // Check legality of parameters
    if ((Smb3A_enable && Smb3B_enable) || (Smb4A_enable && Smb4B_enable))
    {
        return FALSE;
    }

    if (SmbF_mux != SMBF_MUX_NONE)
    {
        switch (SmbF_mux)
        {
            case SMBF_MUX_NONE:
                break;
            case SMBF_MUX_1:
                if (Smb1_enable)
                    return FALSE;
                SmbFifoICU = CHIP_SMB_INT(SMB_MODULE_1);
                break;
            case SMBF_MUX_2:
                if (Smb2_enable)
                    return FALSE;
                SmbFifoICU = CHIP_SMB_INT(SMB_MODULE_2);
                break;
            case SMBF_MUX_3A:
            case SMBF_MUX_3B:
                if (Smb3A_enable || Smb3B_enable)
                    return FALSE;
                SmbFifoICU = CHIP_SMB_INT(SMB_MODULE_3);
                break;
            case SMBF_MUX_4A:
            case SMBF_MUX_4B:
                if (Smb4A_enable || Smb4B_enable)
                    return FALSE;
                SmbFifoICU = CHIP_SMB_INT(SMB_MODULE_4);
                break;
            default:
                return FALSE;
        }
        SET_REG_FIELD(SMB_SEL, SMB_SEL_SMBFSEL, SmbF_mux);
        SMB_Enabled = 1<<SMB_MODULE_5;
    }

    if (Smb1_enable  || SmbF_mux == SMBF_MUX_1)
    {
        if (Smb1_enable)
        {
            SMB_Enabled |= 1<<SMB_MODULE_1;
        }
//        SET_REG_BIT(DEVPU0, SMB_MODULE_1);	//Simon: Using on board pull up register
        SET_REG_FIELD(DEVALT2, DEVALT2_SMB1_SL, TRUE);
    }
    if (Smb2_enable  || SmbF_mux == SMBF_MUX_2)
    {
        if (Smb2_enable)
        {
            SMB_Enabled |= 1<<SMB_MODULE_2;
        }
//        SET_REG_BIT(DEVPU0, SMB_MODULE_2);		//Simon: Using on board pull up register
        SET_REG_FIELD(DEVALT2, DEVALT2_SMB2_SL, TRUE);
    }
    if (Smb3A_enable || SmbF_mux == SMBF_MUX_3A)
    {
        if (Smb3A_enable)
        {
            SMB_Enabled |= 1<<SMB_MODULE_3;
        }
//        SET_REG_BIT(DEVPU0, SMB_MODULE_3);		//Simon: Using on board pull up register
        SET_REG_FIELD(DEVALTA, DEVALTA_SMB3_SL, TRUE);
    }
    if (Smb3B_enable || SmbF_mux == SMBF_MUX_3B)
    {
        if (Smb3B_enable)
        {
            SMB_Enabled |= 1<<SMB_MODULE_3;
            SET_REG_FIELD(SMB_SEL, SMB_SEL_SMB3SEL, 1);
        }
//        SET_REG_BIT(DEVPU0, SMB_MODULE_3);	//Simon: Using on board pull up register
        SET_REG_FIELD(DEVALTC, DEVALTC_SMB3B_SL, TRUE);
    }
    if (Smb4A_enable || SmbF_mux == SMBF_MUX_4A)
    {
        if (Smb4A_enable)
        {
            SMB_Enabled |= 1<<SMB_MODULE_4;
        }
//        SET_REG_BIT(DEVPU0, SMB_MODULE_4);		//Simon: Using on board pull up register
        SET_REG_FIELD(DEVALTA, DEVALTA_SMB4_SL, TRUE);
    }
    if (Smb4B_enable || SmbF_mux == SMBF_MUX_4B)
    {
        if (Smb4B_enable)
        {
            SMB_Enabled |= 1<<SMB_MODULE_4;
            SET_REG_FIELD(SMB_SEL, SMB_SEL_SMB4SEL, 1);
        }
//        SET_REG_BIT(DEVPU0, SMB_MODULE_4);		//Simon: Using on board pull up register
        SET_REG_FIELD(DEVALTC, DEVALTC_SMB4B_SL, TRUE);
    }
    return TRUE;
}
#endif

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_MuxSMB                                                                            */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  smb_module - SMB Module number                                                         */
/*                                                                                                         */
/* Returns:         TRUE  - if this SBMBus was configured via CHIP_SMB_Configure()                         */
/*                  FALSE - otherwise                                                                      */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine checks if the selected SMBx was configured by CHIP_SMB_Configure()        */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (SMB_MODULE_TYPE)
BOOLEAN CHIP_MuxSMB (SMB_MODULE_T smb_module)
{
    if (SMB_Enabled & (1<<smb_module))
        return TRUE;
    return FALSE;
}
#endif

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_GPIO_LowVoltageConfig                                                                           */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio - GPIO number to mux                                                              */
/*                  enable  -    TRUE - Enable, FALSE - Disable                                            */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine enable (or disables) the selected GPIO low voltage (1.8V) support.        */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (GPIO_MODULE_TYPE)
void CHIP_GPIO_LowVoltageConfig (UINT8 gpio, BOOLEAN enable)
{
    volatile UINT8 *reg = 0;
    UINT8  mask = 0;
    switch (gpio)
    {
        /*-------------------------------------------------------------------------------------------------*/
        /* Only the GPIOS that their default mux is not GPIO are different than 0                          */
        /*-------------------------------------------------------------------------------------------------*/
        case 0x01:  
            reg  = (UINT8 *)REG_ADDR(LV_GPIO_CTL2);
            mask = MASK_FIELD(LV_GPIO_CTL2_GPIO01_LV); 
            break;
        case 0x21:
            reg  = (UINT8 *)REG_ADDR(LV_GPIO_CTL0);
            mask = MASK_FIELD(LV_GPIO_CTL0_GPIO21_LV); 
            break;
        case 0x40:
            reg  = (UINT8 *)REG_ADDR(LV_GPIO_CTL0);
            mask = MASK_FIELD(LV_GPIO_CTL0_GPIO40_LV); 
            break;
        case 0x94:
            reg  = (UINT8 *)REG_ADDR(LV_GPIO_CTL0);
            mask = MASK_FIELD(LV_GPIO_CTL0_GPIO94_LV); 
            break;
        default: break;
    }
    if (reg != 0) {
        if (enable) {
            *reg |= mask;
        }
        else {
            *reg &= ~mask;
        }
    }
}
#endif

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_MuxGPIO                                                                           */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio - GPIO number to mux                                                              */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine mux chip pin to function as GPIO                                          */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (GPIO_MODULE_TYPE)
void CHIP_MuxGPIO (UINT8 gpio)
{
    switch (gpio)
    {
        /*-------------------------------------------------------------------------------------------------*/
        /* Only the GPIOS that their default mux is not GPIO are different than 0                          */
        /*-------------------------------------------------------------------------------------------------*/
        case 0x54:  CLEAR_REG_BIT(DEVALT2, 5); break;
        case 0x61:  CLEAR_REG_BIT(DEVALT7, 5); break;
        case 0x62:  CLEAR_REG_BIT(DEVALT7, 4); break;
        case 0x63:  CLEAR_REG_BIT(DEVALT7, 3); break;
        case 0x64:  CLEAR_REG_BIT(DEVALT7, 2); break;
        case 0x86:  CLEAR_REG_BIT(DEVALT8, 2); break;
        default: break;
    }
}
#endif


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_MuxGPIOPort                                                                       */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port      - GPIO port number.                                                          */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine mux chip pins to function as port GPIOs                                   */
/*lint -efunc(715,CHIP_MuxGPIOPort)                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (GPIO_MODULE_TYPE)
void CHIP_MuxGPIOPort (UINT8 port)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* NOT IMPLEMENTED                                                                                     */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(FALSE);
}
#endif

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_MuxMFT                                                                            */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  mft_module - The MFT module to be selected.                                            */
/*                  mft_timer -  The MFT timer to be selected.                                             */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine selects the TXn function to the corresponding pin, where:                 */
/*                   'X' is the MFT timer number  (A/B).                                                  */
/*                   'n' is the MFT module number (1/2/etc').                                             */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (MFT_MODULE_TYPE)
void CHIP_MuxMFT (
    MFT_MODULE_T mft_module,
    MFT_TIMER_T  mft_timer
)
{
    SET_REG_BIT(DEVALT3, (2*mft_module + mft_timer));
}
#endif

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_MuxPS2                                                                            */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  ps2_channel - PS2 channel number                                                       */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine selects the PS2 channel interface on multiplexed pin.                     */
/*                  The function should be called prior to any PS2 operation.                              */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (PS2_MODULE_TYPE)
void CHIP_MuxPS2 (PS2_CHANNEL_T ps2_channel)
{
    ASSERT(ps2_channel < PS2_NUM_OF_CHANNELS);

    switch (ps2_channel)
    {
    case PS2_CHANNEL1:
        SET_REG_FIELD(DEVALT4, DEVALT4_PS2_1_SL, TRUE);
        break;

    case PS2_CHANNEL2:
        SET_REG_FIELD(DEVALT4, DEVALT4_PS2_2_SL, TRUE);
        break;

    case PS2_CHANNEL3:
        SET_REG_FIELD(DEVALT4, DEVALT4_PS2_3_SL, TRUE);
        break;

    default:
        ASSERT(FALSE);
        break;
    }
}
#endif

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_MuxPWM                                                                            */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  pwm_module - The PWM module to be selected.                                            */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine selects the PWMx function to the corresponding pin.                       */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (PWM_MODULE_TYPE)
void CHIP_MuxPWM (PWM_MODULE_T pwm_module)
{
    SET_REG_BIT(DEVALT5, pwm_module);
}
#endif


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_MuxADC                                                                            */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  adc_module - The ADC module to be selected.                                            */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine selects the ADCx function to the corresponding pin.                       */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (ADC_MODULE_TYPE)
void CHIP_MuxADC (ADC_INPUT_T adc_module)
{
    ASSERT(adc_module < ADC_NUM_OF_INPUTS);

    if (adc_module < 8)
    {
        SET_REG_BIT(DEVALT6, adc_module);
    }
    else
    {
        SET_REG_BIT(DEVALTC, (adc_module - 8));
    }
}
#endif


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_MuxDAC                                                                            */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  dac_input - DAC Module number                                                          */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine selects the DACx outputs interface on multiplexed pin.                    */
/*                  The function should be called prior to any other DAC operation.                        */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (DAC_MODULE_TYPE)
void CHIP_MuxDAC (DAC_INPUT_T dac_input)
{
    ASSERT(dac_input < DAC_NUM_OF_INPUTS);

    SET_REG_BIT(DEVALT4, dac_input);
}
#endif


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_MuxPECI                                                                           */
/*                                                                                                         */
/* Parameters:      none                                                                                   */
/* Returns:         TRUE if PECI is enabled; FALSE otherwise.                                              */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine enables the PECI signal for the PECI module (only in PECI bonding option, */
/*                  which is indicated by PECIST internal strap being set to 1).                           */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (PECI_MODULE_TYPE)
BOOLEAN CHIP_MuxPECI (void)
{
    if (! READ_REG_FIELD(SFT_STRP_CFG, SFT_STRP_CFG_PECIST))
    {
        return FALSE;
    }

    SET_REG_FIELD(DEVCNT, DEVCNT_PECI_EN, ENABLE);
    return TRUE;
}
#endif


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        CHIP_MuxSPI                                                                            */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  pwm_module - The PWM module to be selected.                                            */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine selects the PWMx function to the corresponding pin.                       */
/* PC Lint  messages surpression:                                                                          */
/*lint -efunc(715,CHIP_MuxSPI)                                                                             */
/*---------------------------------------------------------------------------------------------------------*/
#if defined (SPI_MODULE_TYPE)
void CHIP_MuxSPI (SPI_MODULE_T spi_module)
{

    /*-----------------------------------------------------------------------------------------------------*/
    /* Select SPI interface                                                                                */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(DEVALT0, DEVALT0_SPI_SL, 1);
}
#endif

