/*---------------------------------------------------------------------------------------------------------*/
/*  Nuvoton Technology Corporation confidential                                                            */
/*                                                                                                         */
/*  Copyright (c) 2010 by Nuvoton Technology Corporation                                                   */
/*  All rights reserved                                                                                    */
/*                                                                                                         */
/*<<<------------------------------------------------------------------------------------------------------*/
/* File Contents:                                                                                          */
/*   gpio_drv.c                                                                                            */
/*            This file contains General Purpose I/O (GPIO) driver implementation                          */
/* Project:                                                                                                */
/*            SWC HAL                                                                                      */
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                                 INCLUDES                                                */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
#include "../../../Chips/chip.h"

#include "gpio_drv.h"
#include "gpio_regs.h"



/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                           INTERFACE FUNCTIONS                                           */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_Init                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio      - GPIO number to configure                                                   */
/*                  gpioDir   - GPIO direction (IN/OUT)                                                    */
/*                  pullDir   - Pull-up/down enabling or disabling                                         */
/*                  vddDriven - Enable if VDD driven                                                       */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine performs GPIO initialization                                              */
/*                                                                                                         */
/*lint -esym(715, vddDriven)                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_Init (
    UINT8           gpio,
    GPIO_DIR_T      gpioDir,
    GPIO_PULL_T     pullDir,
    BOOLEAN         vddDriven
)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Error checking                                                                                      */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(GPIO_GET_PORT_NUM(gpio) < GPIO_NUM_OF_PORTS);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Mux GPIO                                                                                            */
    /*-----------------------------------------------------------------------------------------------------*/
    GPIO_MUX(gpio);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure direction                                                                                 */
    /*-----------------------------------------------------------------------------------------------------*/
    GPIO_ConfigDir(gpio, gpioDir);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure Pull                                                                                      */
    /*-----------------------------------------------------------------------------------------------------*/
    GPIO_ConfigPull(gpio, pullDir);

#if defined GPIO_CAPABILITY_VDD_DRIVEN
    /*-----------------------------------------------------------------------------------------------------*/
    /* Config if VDD driven                                                                                */
    /*-----------------------------------------------------------------------------------------------------*/
    GPIO_ConfigVddDriven(gpio, vddDriven);
#endif
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigDir                                                                         */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio    - GPIO pin number.                                                             */
/*                  gpioDir - GPIO pin direction [INPUT(0) or OUTPUT(1)].                                  */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures the direction (INPUT or OUTPUT) of a given GPIO pin.           */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigDir (
    UINT8       gpio,
    GPIO_DIR_T  gpioDir
)
{
    UINT8 port;
    UINT8 pin;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure the direction of the GPIOx pins according to arguments                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    port = GPIO_GET_PORT_NUM(gpio);
    pin  = GPIO_GET_BIT_NUM(gpio);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Check for Illegal Cases                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure direction                                                                                 */
    /*-----------------------------------------------------------------------------------------------------*/
    if (gpioDir == GPIO_DIR_OUTPUT)
    {
        SET_REG_BIT(PxDIR(port), pin);
    }
    else if (gpioDir == GPIO_DIR_INPUT)
    {
        CLEAR_REG_BIT(PxDIR(port), pin);
    }

#if defined GPIO_CAPABILITY_OPEN_DRAIN
    if (gpioDir == GPIO_DIR_OPEN_DRAIN)
    {
        SET_REG_BIT(PxOTYPE(port), pin);
        SET_REG_BIT(PxDIR(port), pin);
    }
    else
    {
        CLEAR_REG_BIT(PxOTYPE(port), pin);
    }
#endif
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigPull                                                                        */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio    - GPIO pin number.                                                             */
/*                  pullDir - Pull selection [GPIO_PULL_UP (0) or GPIO_PULL_DOWN (1)].                     */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine enables/disables the pull up feature of the given GPIO pin and configures */
/*                  it to pull-up or pull-down (when applicable).                                          */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigPull (
    UINT8       gpio,
    GPIO_PULL_T pullDir
)
{
    UINT8 port;
    UINT8 pin;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Change Pull-Up/Down Enable and Selection according to arguments                                     */
    /*-----------------------------------------------------------------------------------------------------*/
    port = GPIO_GET_PORT_NUM(gpio);
    pin  = GPIO_GET_BIT_NUM(gpio);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Check for Illegal Cases                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure pull enable                                                                               */
    /*-----------------------------------------------------------------------------------------------------*/
    if (pullDir != GPIO_PULL_NONE)
    {
        SET_REG_BIT(PxPULL(port), pin);
    }
    else
    {
        CLEAR_REG_BIT(PxPULL(port), pin);
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure pull dirction                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    if (pullDir == GPIO_PULL_DOWN)
    {
        SET_REG_BIT(PxPUD(port), pin);
    }
    else if (pullDir == GPIO_PULL_UP)
    {
        CLEAR_REG_BIT(PxPUD(port), pin);
    }
}



#if defined GPIO_CAPABILITY_VDD_DRIVEN
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigVddDriven                                                                   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio      - GPIO pin number.                                                           */
/*                  vddDriven - TRUE to enable GPIO by VDD Present Control; FALSE otherwise.               */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures a given GPIO pin to be VDD-driven.                             */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigVddDriven (
    UINT8    gpio,
    BOOLEAN  vddDriven
)
{
    UINT8 port;
    UINT8 pin;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Change Drive Enable by VDD Present according to arguments                                           */
    /*-----------------------------------------------------------------------------------------------------*/
    port = GPIO_GET_PORT_NUM(gpio);
    pin  = GPIO_GET_BIT_NUM(gpio);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Check for Illegal Cases                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set new value                                                                                       */
    /*-----------------------------------------------------------------------------------------------------*/
    if (vddDriven)
    {
        SET_REG_BIT(PxENVDD(port), pin);
    }
    else
    {
        CLEAR_REG_BIT(PxENVDD(port), pin);
    }
}
#endif


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_Read                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio - GPIO pin number.                                                                */
/*                                                                                                         */
/* Returns:         Read value (1 or 0).                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine reads data from a given GPIO pin.                                         */
/*---------------------------------------------------------------------------------------------------------*/
UINT8 GPIO_Read (UINT8 gpio)
{
    UINT8 port;
    UINT8 pin;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Read GPIO pin value according to arguments                                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    port = GPIO_GET_PORT_NUM(gpio);
    pin  = GPIO_GET_BIT_NUM(gpio);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Check for Illegal Cases                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Read PnDIN register                                                                                 */
    /*-----------------------------------------------------------------------------------------------------*/
    return (READ_REG_BIT(PxDIN(port), pin));
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_Write                                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio  - GPIO pin number.                                                               */
/*                  value - Value to be written (0 or 1).                                                  */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine writes data to a given GPIO port.                                         */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_Write (
    UINT8  gpio,
    UINT8  value
)
{
    UINT8 port;
    UINT8 pin;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set the output values of the GPIO pin according to arguments                                        */
    /*-----------------------------------------------------------------------------------------------------*/
    port = GPIO_GET_PORT_NUM(gpio);
    pin  = GPIO_GET_BIT_NUM(gpio);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Check for Illegal Cases                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set GPIO value                                                                                      */
    /*-----------------------------------------------------------------------------------------------------*/
    if (value == 0)
    {
        CLEAR_REG_BIT(PxDOUT(port), pin);
    }
    else
    {
        SET_REG_BIT(PxDOUT(port), pin);
    }
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_InitPort                                                                          */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port          - GPIO port number.                                                      */
/*                  dirMask       - GPIO port direction mask.                                              */
/*                  pullEnMask    - GPIO port pull-up enable mask.                                         */
/*                  pullDirMask   - GPIO port pull-up direction mask.                                      */
/*                  vddDrivenMask - GPIO port VDD Present Control mask.                                    */
/*                  ownerMask     - GPIO port owner mask.                                                  */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine performs GPIO initialization for a given GPIO port.                       */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_InitPort (
    UINT8   port,
    UINT8   dirMask,
    UINT8   pullEnMask,
    UINT8   pullDirMask,
    UINT8   vddDrivenMask
)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Error checking                                                                                      */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set Muxing                                                                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    GPIO_MUX_PORT(port);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure direction                                                                                 */
    /*-----------------------------------------------------------------------------------------------------*/
    GPIO_ConfigPortDir(port, dirMask);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure Pull                                                                                      */
    /*-----------------------------------------------------------------------------------------------------*/
    GPIO_ConfigPortPull(port, pullEnMask, pullDirMask);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Config if VDD driven                                                                                */
    /*-----------------------------------------------------------------------------------------------------*/
    GPIO_ConfigPortVddDriven(port, vddDrivenMask);
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigPortDir                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port    - GPIO port number.                                                            */
/*                  dirMask - GPIO port direction mask.                                                    */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures the direction (INPUT or OUTPUT) of all GPIO pins in a given    */
/*                  GPIO port according to a given direction mask, as follows:                             */
/*                  Bit 'n' with value 0/1 configures GPIO pin 'n' as INPUT/OUTPUT, respectively.          */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigPortDir (
    UINT8   port,
    UINT8   dirMask
)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check for Illegal Cases                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Write PDIR                                                                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    REG_WRITE(PxDIR(port), dirMask);
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigPortPull                                                                    */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port        - GPIO port number.                                                        */
/*                  pullEnMask  - GPIO port pull-up enable mask.                                           */
/*                  pullDirMask - GPIO port pull-up direction mask.                                        */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine enables/disables the pull up feature of all GPIO pins in a given GPIO     */
/*                  port according to a given pull-up enable mask, as follows:                             */
/*                  Bit 'n' with value 0/1 configures GPIO pin 'n' as ENABLE/DISABLE, respectively.        */
/*                                                                                                         */
/*                  It also and configures the pull-up direction (PULL_UP/PULL_DOWN) of all GPIO pins      */
/*                  (when applicable) according to a given pull-up direction mask, as follows:             */
/*                  Bit 'n' with value 0/1 configures GPIO pin 'n' as PULL_UP/PULL_DOWN, respectively.     */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigPortPull (
    UINT8   port,
    UINT8   pullEnMask,
    UINT8   pullDirMask
)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check for Illegal Cases                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Write PnPULL and PnPUD                                                                              */
    /*-----------------------------------------------------------------------------------------------------*/
    REG_WRITE(PxPULL(port), pullEnMask);
    REG_WRITE(PxPUD(port),  pullDirMask);
}



#if defined GPIO_CAPABILITY_VDD_DRIVEN
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigPortVddDriven                                                               */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port          - GPIO port number.                                                      */
/*                  vddDrivenMask - GPIO port VDD Present Control mask.                                    */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures the VDD Presence Control (VDD_DRIVEN or VCC_DRIVEN) of all     */
/*                  GPIO pins in a given GPIO port according to a given VDD Present Control mask, as       */
/*                  follows:                                                                               */
/*                  Bit 'n' with value 0/1 configures GPIO pin 'n' as VCC_DRIVEN/VDD_DRIVEN, respectively. */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigPortVddDriven (
    UINT8   port,
    UINT8   vddDrivenMask
)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check for Illegal Cases                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Write PENVDD                                                                                        */
    /*-----------------------------------------------------------------------------------------------------*/
    REG_WRITE(PxENVDD(port), vddDrivenMask);
}
#endif



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ReadPort                                                                          */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port - GPIO port number.                                                               */
/*                                                                                                         */
/* Returns:         Read value.                                                                            */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine reads data of a given GPIO port.                                          */
/*---------------------------------------------------------------------------------------------------------*/
UINT8 GPIO_ReadPort (UINT8 port)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check for Illegal Cases                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Read PnDIN register                                                                                 */
    /*-----------------------------------------------------------------------------------------------------*/
    return (REG_READ(PxDIN(port)));
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_WritePort                                                                         */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port  - GPIO port number.                                                              */
/*                  value - Value to be written.                                                           */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine writes data to a given GPIO port.                                         */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_WritePort (
    UINT8   port,
    UINT8   value
)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check for Illegal Cases                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT(port < GPIO_NUM_OF_PORTS)

    /*-----------------------------------------------------------------------------------------------------*/
    /* Write PDOUT                                                                                         */
    /*-----------------------------------------------------------------------------------------------------*/
    REG_WRITE(PxDOUT(port), value);
}


