/*---------------------------------------------------------------------------------------------------------*/
/*  Nuvoton Technology Corporation confidential                                                            */
/*                                                                                                         */
/*  Copyright (c) 2010 by Nuvoton Technology Corporation                                                   */
/*  All rights reserved                                                                                    */
/*                                                                                                         */
/*<<<------------------------------------------------------------------------------------------------------*/
/* File Contents:                                                                                          */
/*   gpio_if.h                                                                                             */
/*            This file contains General Purpose I/O (GPIO) interface                                      */
/*  Project:                                                                                               */
/*            SWC HAL                                                                                      */
/*---------------------------------------------------------------------------------------------------------*/
#ifndef _GPIO_IF_H
#define _GPIO_IF_H

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                                 INCLUDES                                                */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
#include "../../Chips/chip.h"

#if defined GPIO_MODULE_TYPE
#include __MODULE_HEADER(gpio, GPIO_MODULE_TYPE)
#endif

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                           TYPES & DEFINITIONS                                           */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* GPIO Direction                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
typedef enum {
    GPIO_DIR_INPUT = 0,
    GPIO_DIR_OUTPUT,
#if defined GPIO_CAPABILITY_OPEN_DRAIN
    GPIO_DIR_OPEN_DRAIN
#endif
} GPIO_DIR_T;

/*---------------------------------------------------------------------------------------------------------*/
/* GPIO Pull-Up/Down                                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
typedef enum {
    GPIO_PULL_NONE = 0,
    GPIO_PULL_UP,
    GPIO_PULL_DOWN
} GPIO_PULL_T;



/*---------------------------------------------------------------------------------------------------------*/
/* Extract Port number and bit number given the GPIO number                                                */
/* The GPIO number is expected to be hexadecimal                                                           */
/*---------------------------------------------------------------------------------------------------------*/
#define GPIO_GET_BIT_NUM(gpio)      LSN(gpio)
#define GPIO_GET_PORT_NUM(gpio)     MSN(gpio)

/*---------------------------------------------------------------------------------------------------------*/
/* Build a GPIO number given its Port and Bit number                                                       */
/* The GPIO number is represented in hexadecimal                                                           */
/*---------------------------------------------------------------------------------------------------------*/
#define GPIO_NUM(port, bit)         ((port)*0x10 + (bit))

/*---------------------------------------------------------------------------------------------------------*/
/* Specify GPIO Port Number (used only in GPIO Port operations)                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define GPIO_PORT(port)             (port)



/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                           INTERFACE FUNCTIONS                                           */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_Init                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio      - GPIO number to configure                                                   */
/*                  gpioDir   - GPIO direction (IN/OUT)                                                    */
/*                  pullDir   - Pull-up/down enabling or disabling                                         */
/*                  vddDriven - Enable if VDD driven                                                       */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine performs GPIO initialization                                              */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_Init (UINT8 gpio, GPIO_DIR_T gpioDir, GPIO_PULL_T pullDir, BOOLEAN vddDriven);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigDir                                                                         */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio     - GPIO number.                                                                */
/*                  gpioDir  - GPIO direction [INPUT(0) or OUTPUT(1)].                                     */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures the direction (INPUT or OUTPUT) of a given GPIO pin.           */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigDir (UINT8 gpio, GPIO_DIR_T gpioDir);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigPull                                                                        */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio     - GPIO number.                                                                */
/*                  pullDir  - Pull selection [GPIO_PULL_NONE (0) or GPIO_PULL_UP(1) or GPIO_PULL_DOWN(2)] */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine enables/disables the pull up feature of the given GPIO pin and configures */
/*                  it to pull-up or pull-down (when applicable).                                          */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigPull (UINT8 gpio, GPIO_PULL_T pullDir);

#if defined GPIO_CAPABILITY_VDD_DRIVEN
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigVddDriven                                                                   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio       - GPIO number.                                                              */
/*                  vddDriven  - TRUE to enable GPIO by VDD Present Control; FALSE otherwise.              */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures a given GPIO pin to be VDD-driven.                             */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigVddDriven (UINT8 gpio, BOOLEAN vddDriven);
#endif

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_Read                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio - GPIO number.                                                                    */
/*                                                                                                         */
/* Returns:         Read value (1 or 0).                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine reads data from a given GPIO pin.                                         */
/*---------------------------------------------------------------------------------------------------------*/
UINT8 GPIO_Read (UINT8 gpio);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_Write                                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  gpio     - GPIO number.                                                                */
/*                  pin_data - Value to be written (0 or 1).                                               */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine writes data to a given GPIO port.                                         */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_Write (UINT8 gpio, UINT8 pin_data);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_InitPort                                                                          */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port          - GPIO port number.                                                      */
/*                  dirMask       - GPIO port direction mask.                                              */
/*                  pullEnMask    - GPIO port pull-up enable mask.                                         */
/*                  pullDirMask   - GPIO port pull-up direction mask.                                      */
/*                  vddDrivenMask - GPIO port VDD Present Control mask.                                    */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine performs GPIO initialization for a given GPIO port.                       */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_InitPort (UINT8 port, UINT8 dirMask, UINT8 pullEnMask, UINT8 pullDirMask, UINT8 vddDrivenMask);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigPortDir                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port    - GPIO port number.                                                            */
/*                  dirMask - GPIO port direction mask.                                                    */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures the direction (INPUT or OUTPUT) of all GPIO pins in a given    */
/*                  GPIO port according to a given direction mask, as follows:                             */
/*                  Bit 'n' with value 0/1 configures GPIO pin 'n' as INPUT/OUTPUT, respectively.          */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigPortDir (UINT8 port, UINT8 dirMask);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigPortPull                                                                    */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port        - GPIO port number.                                                        */
/*                  pullEnMask  - GPIO port pull-up enable mask.                                           */
/*                  pullDirMask - GPIO port pull-up direction mask.                                        */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine enables/disables the pull up feature of all GPIO pins in a given GPIO     */
/*                  port according to a given pull-up enable mask, as follows:                             */
/*                  Bit 'n' with value 0/1 configures GPIO pin 'n' as ENABLE/DISABLE, respectively.        */
/*                                                                                                         */
/*                  It also and configures the pull-up direction (PULL_UP/PULL_DOWN) of all GPIO pins      */
/*                  (when applicable) according to a given pull-up direction mask, as follows:             */
/*                  Bit 'n' with value 0/1 configures GPIO pin 'n' as PULL_UP/PULL_DOWN, respectively.     */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigPortPull (UINT8 port, UINT8 pullEnMask, UINT8 pullDirMask);

#if defined GPIO_CAPABILITY_VDD_DRIVEN
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ConfigPortVddDriven                                                               */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port          - GPIO port number.                                                      */
/*                  vddDrivenMask - GPIO port VDD Present Control mask.                                    */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures the VDD Presence Control (VDD_DRIVEN or VCC_DRIVEN) of all     */
/*                  GPIO pins in a given GPIO port according to a given VDD Present Control mask, as       */
/*                  follows:                                                                               */
/*                  Bit 'n' with value 0/1 configures GPIO pin 'n' as VCC_DRIVEN/VDD_DRIVEN, respectively. */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_ConfigPortVddDriven (UINT8 port, UINT8 vddDrivenMask);
#endif

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_ReadPort                                                                          */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port - GPIO port number.                                                               */
/*                                                                                                         */
/* Returns:         Read value.                                                                            */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine reads data of a given GPIO port.                                          */
/*---------------------------------------------------------------------------------------------------------*/
UINT8 GPIO_ReadPort (UINT8 port);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        GPIO_WritePort                                                                         */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  port  - GPIO port number.                                                              */
/*                  value - Value to be written.                                                           */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine writes data to a given GPIO port.                                         */
/*---------------------------------------------------------------------------------------------------------*/
void GPIO_WritePort (UINT8 port, UINT8 value);

#endif  /* _GPIO_IF_H */

