/*---------------------------------------------------------------------------------------------------------*/
/*  Nuvoton Technology Corporation confidential                                                            */
/*                                                                                                         */
/*  Copyright (c) 2010 by Nuvoton Technology Corporation                                                   */
/*  All rights reserved                                                                                    */
/*                                                                                                         */
/*<<<------------------------------------------------------------------------------------------------------*/
/* File Contents:                                                                                          */
/*   hfcg_drv.c                                                                                            */
/*            This file contains High Frequency Clock Generator (HFCG) driver implementation               */
/* Project:                                                                                                */
/*            SWC HAL                                                                                      */
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                                 INCLUDES                                                */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
#include "../../../Chips/chip.h"

#include "hfcg_drv.h"
#include "hfcg_regs.h"

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                  LOCAL FUNCTIONS FORWARD DECLARATIONS                                   */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
static DEFS_STATUS  HFCG_SetFreqMultClkFreq_L   (UINT16 m, UINT8 n);
static DEFS_STATUS  HFCG_SetFlashClkFreq_L      (UINT8 flashDiv);
static DEFS_STATUS  HFCG_SetCoreClkFreq_L       (UINT8 coreDiv);
#ifdef HFCG_CAPABILITY_APB_CLOCK
static DEFS_STATUS  HFCG_SetAPBClkFreq_L        (HFCG_ABP_BUS_T bus, UINT8 apbDiv);
#endif
#ifdef HFCG_CAPABILITY_SIO_CLOCK
static DEFS_STATUS HFCG_SetSIOClkFreq_L         (UINT8 sioDiv);
#endif

static void         HFCG_DelayMicroSec_L        (UINT32 delay, UINT32 coreClk);

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                               DEFINITIONS                                               */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/



/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                           INTERFACE FUNCTIONS                                           */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_Config                                                                            */
/*                                                                                                         */
/* Parameters:      none                                                                                   */
/* Returns:         DEFS_STATUS_OK if no error occurred or error code otherwise                            */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures HFCG clock values to defaults set by <CHIP>.h file             */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_Config (void)
{
    UINT8   n           = (UINT8)(FMCLK_HFCGN);
    UINT16  m           = (UINT16)(MAKE16(FMCLK_HFCGML, FMCLK_HFCGMH));
    UINT8   coreDiv     = (UINT8)(CORE_DIVIDER);
    UINT8   flashDiv    = (UINT8)(FLASH_DIVIDER);

    return HFCG_ConfigClocks(m, n, coreDiv, flashDiv);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_ConfigClocks                                                                      */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  freqMulM    - HFCG FMUL M parameter                                                    */
/*                  freqMulN    - HFCG FMUL N parameter                                                    */
/*                  flashDiv    - Flash divisor                                                            */
/*                  coreDiv     - Core divisor                                                             */
/*                                                                                                         */
/* Returns:         DEFS_STATUS_OK if no error occurred or error code otherwise                            */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures HFCG clocks                                                    */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_ConfigClocks(UINT16 freqMulM, UINT8 freqMulN, UINT8 coreDiv, UINT8 flashDiv)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure FMCLK                                                                                     */
    /*-----------------------------------------------------------------------------------------------------*/
    DEFS_STATUS_RET_CHECK(HFCG_SetFreqMultClkFreq_L(freqMulM, freqMulN));

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure Core Clock                                                                                */
    /*-----------------------------------------------------------------------------------------------------*/
    DEFS_STATUS_RET_CHECK(HFCG_SetCoreClkFreq_L(coreDiv));

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure Flash Clock                                                                               */
    /*-----------------------------------------------------------------------------------------------------*/
    DEFS_STATUS_RET_CHECK(HFCG_SetFlashClkFreq_L(flashDiv));

    return DEFS_STATUS_OK;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_Lock                                                                              */
/*                                                                                                         */
/* Parameters:      none                                                                                   */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine disables write access to all HFCG registers except HFCGP register.        */
/*                  Only reset clears this bit.                                                            */
/*---------------------------------------------------------------------------------------------------------*/
void HFCG_Lock (void)
{
    SET_REG_FIELD(HFCGCTRL, HFCGCTRL_LOCK, TRUE);
}


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_SetFreqMultClkFreq                                                                */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  fmFreq - Frequency Multiplier reqeusted frequency                                      */
/*                                                                                                         */
/* Returns:         DEFS_STATUS_OK if no error occurred or error code otherwise                            */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine sets new Frequency Multiplier frequency                                   */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_SetFreqMultClkFreq(UINT32 fmFreq)
{
    UINT8   n;
    UINT16  m;

    /*-----------------------------------------------------------------------------------------------------*/
    /* By default we set N to 2                                                                            */
    /*-----------------------------------------------------------------------------------------------------*/
    n = 2;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Calculate M value                                                                                   */
    /*-----------------------------------------------------------------------------------------------------*/
    m = (UINT16)(DIV_ROUND(fmFreq, LFCG_CORE_CLK) * n);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure FMUL                                                                                      */
    /*-----------------------------------------------------------------------------------------------------*/
    return HFCG_SetFreqMultClkFreq_L(m, n);
}


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_SetFlashClkFreq                                                                   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  flashClk - Required Flash Clock                                                        */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures flash clock                                                    */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_SetFlashClkFreq (UINT32 flashClk)
{
    UINT32 fmclk    = 0;
    UINT8  flashDiv = 0;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Get FMCLK frequency                                                                                 */
    /*-----------------------------------------------------------------------------------------------------*/
    fmclk = HFCG_GetFreqMultClkFreq();

    /*-----------------------------------------------------------------------------------------------------*/
    /* Calculate Flash Divisor                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    flashDiv = (UINT8)DIV_ROUND(fmclk, flashClk);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure Flash Clock                                                                               */
    /*-----------------------------------------------------------------------------------------------------*/
    return HFCG_SetFlashClkFreq_L(flashDiv);
}


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_SetCoreClkFreq                                                                    */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  coreClk - Required Core Clock                                                          */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures core clock                                                     */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_SetCoreClkFreq (UINT32 coreClk)
{
    UINT8  coreDiv  = 0;
    UINT32 flashClk = 0;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Get Flash CLK                                                                                       */
    /*-----------------------------------------------------------------------------------------------------*/
    flashClk = HFCG_GetFlashClkFreq();

    /*-----------------------------------------------------------------------------------------------------*/
    /* Calculate core divisor                                                                              */
    /*-----------------------------------------------------------------------------------------------------*/
    coreDiv = (UINT8)DIV_ROUND(flashClk, coreClk);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set core divisor                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    return HFCG_SetCoreClkFreq_L(coreDiv);
}


#ifdef HFCG_CAPABILITY_APB_CLOCK
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_SetAPBClkFreq                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  bus     - APB Bus number                                                               */
/*                  apbClk  - Required APB Clock                                                           */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures APB clock                                                      */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_SetAPBClkFreq (HFCG_ABP_BUS_T bus, UINT32 apbClk)
{
    UINT32 coreCLK = 0;
    UINT8  apbDiv  = 0;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Get CLK frequency (The APB clock is derived from Core clock (CLK))                                  */
    /*-----------------------------------------------------------------------------------------------------*/
    coreCLK = HFCG_GetCoreClkFreq();

    /*-----------------------------------------------------------------------------------------------------*/
    /* Calculate APB divisor                                                                              */
    /*-----------------------------------------------------------------------------------------------------*/
    apbDiv = (UINT8)DIV_ROUND(coreCLK, apbClk);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set core divisor                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    return HFCG_SetAPBClkFreq_L(bus, apbDiv);
}
#endif


#ifdef HFCG_CAPABILITY_SIO_CLOCK
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_SetSIOClkFreq                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  sioClk  - Required Super I/O Clock                                                     */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures Super I/O clock                                                */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_SetSIOClkFreq (UINT32 sioClk)
{
    UINT32 fmclk = 0;
    UINT8  sioDiv  = 0;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Get FMCLK frequency (The Super I/O clock (SIOCLK) is derived from FMCLK)                            */
    /*-----------------------------------------------------------------------------------------------------*/
    fmclk = HFCG_GetFreqMultClkFreq();

    /*-----------------------------------------------------------------------------------------------------*/
    /* Calculate SIO divisor                                                                               */
    /*-----------------------------------------------------------------------------------------------------*/
    sioDiv = (UINT8)DIV_ROUND(fmclk, sioClk);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set core divisor                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    return HFCG_SetSIOClkFreq_L(sioDiv);
}
#endif



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_GetFreqMultClkFreq                                                                */
/*                                                                                                         */
/* Parameters:      none                                                                                   */
/* Returns:         Frequency multiplier clock (FMCLK) frequency, in 1 MHz resolution.                     */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine retrieves the frequency multiplier clock FMCLK, which is based on the     */
/*                  32 KHz Clock domain (32.768 KHz, low frequency (LFCLK) clock signal).                  */
/*                  FMCLK is rounded to 1 MHz resolution.                                                  */
/*---------------------------------------------------------------------------------------------------------*/
UINT32 HFCG_GetFreqMultClkFreq (void)
{
    UINT8   N, division;
    UINT16  M;
    UINT32  freq, residue;

    N = READ_REG_FIELD(HFCGN, HFCGN_HFCGN5_0);
    M = MAKE16(REG_READ(HFCGML), REG_READ(HFCGMH));

    /*-----------------------------------------------------------------------------------------------------*/
    /* FMCLK = LFCG_CORE_CLK * (M / N)                                                                     */
    /*-----------------------------------------------------------------------------------------------------*/
    freq = ((UINT32)LFCG_CORE_CLK * M) / N;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Round frequency to a resolution of _1MHz_                                                           */
    /*-----------------------------------------------------------------------------------------------------*/
    division    = (UINT8)(freq / _1MHz_);
    residue     = freq % _1MHz_;

    /*-----------------------------------------------------------------------------------------------------*/
    /* In case the remainder is more than 50%, round up the division                                       */
    /*-----------------------------------------------------------------------------------------------------*/
    if (residue >= ((UINT32)_1MHz_/2))
    {
        division += 1;
    }

    return (division * _1MHz_);
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_GetFlashClkFreq                                                                   */
/*                                                                                                         */
/* Parameters:      none                                                                                   */
/* Returns:         Flash clock (FCLK) frequency                                                           */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine retrieves the the flash clock (FCLK), which is based on the               */
/*                  high-frequency clock FMCLK.                                                            */
/*---------------------------------------------------------------------------------------------------------*/
UINT32 HFCG_GetFlashClkFreq (void)
{
    UINT32  FMCLK, FCLK;
    UINT8   FPRED;

    /*-----------------------------------------------------------------------------------------------------*/
    /* The flash clock (FCLK) is derived from FMCLK via a 4-bit prescaler (FPRE) as follows:               */
    /* FCLK = FMCLK / (FPRED + 1)                                                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    FMCLK   = HFCG_GetFreqMultClkFreq();
    FPRED   = READ_REG_FIELD(HFCGP, HFCGP_FPRED) + 1;
    FCLK    = FMCLK / FPRED;

    return FCLK;
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_GetCoreClkFreq                                                                    */
/*                                                                                                         */
/* Parameters:      none                                                                                   */
/* Returns:         Core clock (CLK) frequency                                                             */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine retrieves the core clock (CLK), which is based on the Flash clock FCLK.   */
/*---------------------------------------------------------------------------------------------------------*/
UINT32 HFCG_GetCoreClkFreq (void)
{
    UINT32  FCLK, CLK;
    UINT8   CPRED;

    /*-----------------------------------------------------------------------------------------------------*/
    /* The Core clock (CLK) is derived from FCLK via a 4-bit prescaler (CPRE) as follows:                  */
    /* CLK = FCLK / (CPRED + 1)                                                                            */
    /*-----------------------------------------------------------------------------------------------------*/
    FCLK    = HFCG_GetFlashClkFreq();
    CPRED   = READ_REG_FIELD(HFCGP, HFCGP_CPRED) + 1;
    CLK     = FCLK / CPRED;

    return CLK;
}


#ifdef HFCG_CAPABILITY_APB_CLOCK
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_GetAPBClkFreq                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  bus     - APB Bus number                                                               */
/*                                                                                                         */
/* Returns:         APB clock (APBCLK) frequency                                                           */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine retrieves the APB clock (APBCLK), which is based on CLK.                  */
/*---------------------------------------------------------------------------------------------------------*/
UINT32 HFCG_GetAPBClkFreq (HFCG_ABP_BUS_T bus)
{
    UINT32  APBCLK, CLK;
    UINT8   APBDIV;

    /*-----------------------------------------------------------------------------------------------------*/
    /* The APB clock (APBCLK) is derived from CLK via a 2-bit prescaler (APBDIV) as follows:               */
    /* APBCLK = CLK / (APBDIV + 1)                                                                         */
    /*-----------------------------------------------------------------------------------------------------*/
    CLK     = HFCG_GetCoreClkFreq();
    APBDIV	= READ_REG_FIELD(HFCBCD, HFCBCD_APBDIV(bus)) + 1;
	
    APBCLK  = CLK / APBDIV;

    return APBCLK;
}
#endif


#ifdef HFCG_CAPABILITY_SIO_CLOCK
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_GetSIOClkFreq                                                                     */
/*                                                                                                         */
/* Parameters:      none                                                                                   */
/* Returns:         Super I/O clock (SIOCLK) frequency                                                     */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine retrieves the Super I/O clock (SIOCLK), which is based on FMCLK.          */
/*---------------------------------------------------------------------------------------------------------*/
UINT32 HFCG_GetSIOClkFreq (void)
{
    UINT32  SIOCLK, FMCLK;
    UINT8   SIODIV;

    /*-----------------------------------------------------------------------------------------------------*/
    /* The Super I/O clock (SIOCLK) is derived from FMCLK via a 1-bit prescaler (SIODIV) as follows:       */
    /* SIOCLK = FMCLK / (SIODIV + 1)                                                                       */
    /*-----------------------------------------------------------------------------------------------------*/
    FMCLK   = HFCG_GetFreqMultClkFreq();
    SIODIV  = READ_REG_FIELD(HFCBCD, HFCBCD_SIODIV) + 1;
    SIOCLK  = FMCLK / SIODIV;

    return SIOCLK;
}
#endif


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_DelayMicroSec_DefaultCoreClock                                                    */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  microSecDelay - Delay value in microseconds                                            */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine performs delay based on Default Core Clock configuration from <CHIP>.h    */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_DelayMicroSec_DefaultCoreClock(UINT32 microSecDelay)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check if we exceed the limit                                                                        */
    /*                                                                                                     */
    /*lint -e{774,506}                                                                                     */
    /*-----------------------------------------------------------------------------------------------------*/
    if ((HFCG_CORE_CLK  < _1MHz_) ||
       ( microSecDelay  >  (0xFFFFFFFF / (HFCG_CORE_CLK / _1SEC_IN_USEC_))))
    {
        return DEFS_STATUS_PARAMETER_OUT_OF_RANGE;
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Execute delay                                                                                       */
    /*-----------------------------------------------------------------------------------------------------*/
    HFCG_DelayMicroSec_L(microSecDelay, HFCG_CORE_CLK);

    return DEFS_STATUS_OK;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_DelayMicroSec_CurrentCoreClock                                                    */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  microSecDelay - Delay value in miscroseconds                                           */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine performs delay based on current core clock                                */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_DelayMicroSec_CurrentCoreClock(UINT32 microSecDelay)
{
    return HFCG_DelayMicroSec_ProvidedCoreClock(microSecDelay, HFCG_GetCoreClkFreq());
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_DelayMicroSec_ProvidedCoreClock                                                   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  microSecDelay   - Delay value in miscorseconds                                         */
/*                  coreClk         - Core clock value in Hz                                               */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine performs delay based on provided core clock                               */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_DelayMicroSec_ProvidedCoreClock(UINT32 microSecDelay, UINT32 coreClk)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check if we exceed the limit                                                                        */
    /*-----------------------------------------------------------------------------------------------------*/
    if ((coreClk        < _1MHz_) ||
       ( microSecDelay  >  (0xFFFFFFFF / (coreClk / _1SEC_IN_USEC_))))
    {
        return DEFS_STATUS_PARAMETER_OUT_OF_RANGE;
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Execute delay                                                                                       */
    /*-----------------------------------------------------------------------------------------------------*/
    HFCG_DelayMicroSec_L(microSecDelay, coreClk);

    return DEFS_STATUS_OK;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_ConvertMicroSecToIterations                                                       */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  microSecDelay   - Delay in microseconds                                                */
/*                  coreClk         - Core clock value in Hz                                               */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine convert microsecond delay into loop iterations                            */
/*---------------------------------------------------------------------------------------------------------*/
UINT32 HFCG_ConvertMicroSecToIterations(UINT32 microSecDelay, UINT32 coreClk)
{

    /*-----------------------------------------------------------------------------------------------------*/
    /* Calculate number of iterations:                                                                     */
    /*                                                                                                     */
    /*  iterations =    coreClk        microSecDelay                                                       */
    /*               -------------  X  -------------                                                       */
    /*              CyclesPerIter      _1SEC_IN_USEC_                                                      */
    /*-----------------------------------------------------------------------------------------------------*/
    return (UINT32)(((coreClk / _1SEC_IN_USEC_) * microSecDelay) / HFCG_NUM_OF_CYCLES_PER_ITERATION);
}




/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                             LOCAL FUNCTIONS                                             */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_SetFreqMultClkFreq_L                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  m   - HFCG M value                                                                     */
/*                  n   - HFCG N value                                                                     */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine sets new Frequency Multiplier frequency                                   */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_SetFreqMultClkFreq_L(UINT16 m, UINT8 n)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure Frequency multiplier values according to the requested FMCLK Clock Frequency              */
    /*-----------------------------------------------------------------------------------------------------*/
    REG_WRITE(HFCGN,  n);
    REG_WRITE(HFCGML, LSB(m));
    REG_WRITE(HFCGMH, MSB(m));

    /*-----------------------------------------------------------------------------------------------------*/
    /* Load M and N values into the frequency multiplier                                                   */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(HFCGCTRL, HFCGCTRL_LOAD, TRUE);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Wait for stable high-frequency clock:                                                               */
    /* The HFCG performs a fast frequency change to approach the new set frequency value.                  */
    /* During this time, the Clock Valid indication holds the clock outputs FMCLK and CLK high and FCLK    */
    /* low. This starts up to 3 x 32K (LFCLK) clocks, which is estimated as ~100us, after the LOAD bit was */
    /* set to 1 and can take up to 3 ms to complete.                                                       */
    /*-----------------------------------------------------------------------------------------------------*/
#ifdef HFCG_CAPABILITY_CLOCK_CHANGING_BIT
    BUSY_WAIT_TIMEOUT(READ_REG_FIELD(HFCGCTRL, HFCGCTRL_CLK_CHNG) == 1, 0x8000);
#else
    DELAY_LOOP(16);
#endif

    return DEFS_STATUS_OK;
}


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_SetFlashClkFreq_L                                                                 */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  flashDiv - Flash clock divisor                                                         */
/*                                                                                                         */
/* Returns:         DEFS_STATUS_OK on success, error code otherwise                                        */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures flash clock                                                    */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_SetFlashClkFreq_L(UINT8 flashDiv)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check parameters                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    DEFS_STATUS_COND_CHECK(flashDiv != 0, DEFS_STATUS_PARAMETER_OUT_OF_RANGE);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure Flash Prescaler Divider Value                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(HFCGP, HFCGP_FPRED, (flashDiv - 1));

    /*-----------------------------------------------------------------------------------------------------*/
    /* Wait at least 16 CLK cycles, for Flash frequency to change                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    DELAY_LOOP(16);

    return DEFS_STATUS_OK;
}


/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_SetCoreClkFreq_L                                                                  */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  coreDiv - Core clock divisor                                                           */
/*                                                                                                         */
/* Returns:         DEFS_STATUS_OK on success, error code otherwise                                        */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures core clock                                                     */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_SetCoreClkFreq_L (UINT8 coreDiv)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check parameters                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    DEFS_STATUS_COND_CHECK(coreDiv != 0, DEFS_STATUS_PARAMETER_OUT_OF_RANGE);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure Core Prescaler Divider Value                                                              */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(HFCGP, HFCGP_CPRED, coreDiv  - 1);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Wait at least 16 CLK cycles, for CLK frequency to change                                            */
    /*-----------------------------------------------------------------------------------------------------*/
    DELAY_LOOP(16);

    return DEFS_STATUS_OK;
}


#ifdef HFCG_CAPABILITY_APB_CLOCK
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_SetAPBClkFreq_L                                                                   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  bus     - APB Bus number                                                               */
/*                  apbDiv  - APB clock divisor                                                            */
/*                                                                                                         */
/* Returns:         DEFS_STATUS_OK on success, error code otherwise                                        */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures APB clock                                                      */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_SetAPBClkFreq_L (HFCG_ABP_BUS_T bus, UINT8 apbDiv)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check parameters                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    DEFS_STATUS_COND_CHECK(apbDiv != 0, DEFS_STATUS_PARAMETER_OUT_OF_RANGE);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure APB Prescaler Divider Value                                                               */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(HFCBCD, HFCBCD_APBDIV(bus), apbDiv - 1);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Wait at least 16 CLK cycles, for APB frequency to change                                            */
    /*-----------------------------------------------------------------------------------------------------*/
    DELAY_LOOP(16);

    return DEFS_STATUS_OK;
}
#endif


#ifdef HFCG_CAPABILITY_SIO_CLOCK
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_SetSIOClkFreq_L                                                                   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  sioDiv - Super I/O clock divisor                                                       */
/*                                                                                                         */
/* Returns:         DEFS_STATUS_OK on success, error code otherwise                                        */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures Super I/O clock                                                */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS HFCG_SetSIOClkFreq_L (UINT8 sioDiv)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Check parameters                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    DEFS_STATUS_COND_CHECK(sioDiv != 0, DEFS_STATUS_PARAMETER_OUT_OF_RANGE);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure SIO Prescaler Divider Value                                                               */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(HFCBCD, HFCBCD_SIODIV, sioDiv - 1);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Wait at least 16 CLK cycles, for SIO frequency to change                                            */
    /*-----------------------------------------------------------------------------------------------------*/
    DELAY_LOOP(16);

    return DEFS_STATUS_OK;
}
#endif



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        HFCG_DelayMicroSec_L                                                                   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  microSecDelay   - Delay in microseconds                                                */
/*                  coreClk         - Core clock value in Hz                                               */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine executes microsecond delay (without error checking)                       */
/*---------------------------------------------------------------------------------------------------------*/
void HFCG_DelayMicroSec_L(UINT32 microSecDelay, UINT32 coreClk)
{
    volatile UINT32   i;
    volatile UINT32   iterations;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Calculate number of iterations:                                                                     */
    /*                                                                                                     */
    /*  iterations =    coreClk        microSecDelay                                                       */
    /*               -------------  X  -------------                                                       */
    /*              CyclesPerIter      _1SEC_IN_USEC_                                                      */
    /*-----------------------------------------------------------------------------------------------------*/
    iterations = HFCG_ConvertMicroSecToIterations(microSecDelay, coreClk);

    /*-----------------------------------------------------------------------------------------------------*/
    /* The actual wait loop                                                                                */
    /*-----------------------------------------------------------------------------------------------------*/
    for (i = 0; i < iterations; i++) ;
}

