/*---------------------------------------------------------------------------------------------------------*/
/*  Nuvoton Technology Corporation confidential                                                            */
/*                                                                                                         */
/*  Copyright (c) 2010 by Nuvoton Technology Corporation                                                   */
/*  All rights reserved                                                                                    */
/*                                                                                                         */
/*<<<------------------------------------------------------------------------------------------------------*/
/* File Contents:                                                                                          */
/*   icu_if.h                                                                                              */
/*            This file contains Interrupt Control Unit (ICU) interface                                    */
/*  Project:                                                                                               */
/*            SWC HAL                                                                                      */
/*---------------------------------------------------------------------------------------------------------*/
#ifndef _ICU_IF_H
#define _ICU_IF_H

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                                 INCLUDES                                                */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
#include "../../Chips/chip.h"

#if defined ICU_MODULE_TYPE
#include __MODULE_HEADER(icu, ICU_MODULE_TYPE)
#endif

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                           TYPES & DEFINITIONS                                           */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* Trap Mapping                                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
typedef enum
{
    ICU_TRAP_RES0   = 0,    /* Reserved                     */
    ICU_TRAP_NMI    = 1,    /* Non-Maskable Interrupt       */
    ICU_TRAP_RES2   = 2,    /* Reserved                     */
    ICU_TRAP_RES3   = 3,    /* Reserved                     */
    ICU_TRAP_RES4   = 4,    /* Reserved                     */
    ICU_TRAP_SVC    = 5,    /* Supervisor Call Trap         */
    ICU_TRAP_DVZ    = 6,    /* Divide by Zero Trap          */
    ICU_TRAP_FLG    = 7,    /* Flag Trap                    */
    ICU_TRAP_BPT    = 8,    /* Breakpoint Trap              */
    ICU_TRAP_TRC    = 9,    /* Trace Trapt                  */
    ICU_TRAP_UND    = 10,   /* Undefined Instruction Trap   */
    ICU_TRAP_RES11  = 11,   /* Reserved                     */
    ICU_TRAP_IAD    = 12,   /* Illegal Address Trap         */
    ICU_TRAP_RES13  = 13,   /* Reserved                     */
    ICU_TRAP_DBG    = 14,   /* Debug Trap                   */
    ICU_TRAP_ISE    = 15    /* In-System Emulator Interrupt */
} ICU_TRAP_SRC_T;

/*---------------------------------------------------------------------------------------------------------*/
/* Interrupt Mapping                                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
typedef enum
{
    ICU_INT_0   = 0,
    ICU_INT_1,
    ICU_INT_2,
    ICU_INT_3,
    ICU_INT_4,
    ICU_INT_5,
    ICU_INT_6,
    ICU_INT_7,
    ICU_INT_8,
    ICU_INT_9,
    ICU_INT_10,
    ICU_INT_11,
    ICU_INT_12,
    ICU_INT_13,
    ICU_INT_14,
    ICU_INT_15,
    ICU_INT_16,
    ICU_INT_17,
    ICU_INT_18,
    ICU_INT_19,
    ICU_INT_20,
    ICU_INT_21,
    ICU_INT_22,
    ICU_INT_23,
    ICU_INT_24,
    ICU_INT_25,
    ICU_INT_26,
    ICU_INT_27,
    ICU_INT_28,
    ICU_INT_29,
    ICU_INT_30,
    ICU_INT_31,

    ICU_INT_UNDEF = 0xFF
} ICU_INT_SRC_T;


/*---------------------------------------------------------------------------------------------------------*/
/* Interrupt Dispatch Table Size (includes all traps and interrupts)                                       */
/*---------------------------------------------------------------------------------------------------------*/
#define ICU_TABLE_SIZE          (ICU_TRAP_NUM + ICU_INTERRUPT_NUM)

/*---------------------------------------------------------------------------------------------------------*/
/* First table enteries are dedicated for traps, followed by interrupts                                    */
/*---------------------------------------------------------------------------------------------------------*/
#define ICU_INT_BASE            ICU_TRAP_NUM

#ifdef EXTERNAL_DISPATCH_TABLE
#define STATIC_DISPATCH_TABLE       /* External dispatch table must be static */
#endif

#ifdef STATIC_DISPATCH_TABLE
/*---------------------------------------------------------------------------------------------------------*/
/* Allow the same code for static dispatch table (handler installation is ignored)                         */
/*---------------------------------------------------------------------------------------------------------*/
#define     ICU_InstallHandler(INT_NO, PROC)    {}
#define     ICU_InstallTrap(TRAP_NO, PROC)      {}
#endif

typedef void (*HANDLER_T)(UINT16 int_num);

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                           INTERFACE FUNCTIONS                                           */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ICU_Init                                                                               */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  inttable - TRUE: Install null_handlers for all interrupts                              */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:    This function only sets the Global Interrupt Enable (I bit in PSR register).           */
/*                  The Local Interrupt Enable (E bit in PSR register) remains cleared.                    */
/*                  The upper-layer should call ENABLE_INTERRUPTS to set the E bit when desired.           */
/* Description:                                                                                            */
/*                  This routine:                                                                          */
/*                  1. Masks all interrupts.                                                               */
/*                  2. Clears all pending interrupts.                                                      */
/*                  3. if inttable is TRUE: Installs the default interrupt dispatch-table                  */
/*                     (null_handlers for all interrupts).                                                 */
/*---------------------------------------------------------------------------------------------------------*/
void ICU_Init (BOOLEAN inttable);

#ifndef EXTERNAL_DISPATCH_TABLE
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ICU_InitTab                                                                            */
/*                                                                                                         */
/* Parameters:      none                                                                                   */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine initializes the dispatch-table (interrupt-vector).                        */
/*---------------------------------------------------------------------------------------------------------*/
void ICU_InitTab (void);

#endif /* EXTERNAL_DISPATCH_TABLE */

#ifndef STATIC_DISPATCH_TABLE
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ICU_InstallTrap                                                                        */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  trap_no - number of trap for which the handler should be installed.                    */
/*                  proc   - trap handler procedure to be installed.                                       */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine installs the provided trap handler in the dispatch table.                 */
/*                  The function is used to install or change the trap handler when the                    */
/*                  dispatch-table is placed in RAM.                                                       */
/*---------------------------------------------------------------------------------------------------------*/
void ICU_InstallTrap (ICU_TRAP_SRC_T trap_no, HANDLER_T proc);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ICU_InstallHandler                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  int_no - number of interrupt for which the handler should be installed.                */
/*                  proc   - interrupt handler procedure to be installed.                                  */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine installs the provided interrupt handler in the dispatch table.            */
/*                  The function is used to install or change the interrupt handler when the               */
/*                  dispatch-table is placed in RAM.                                                       */
/*---------------------------------------------------------------------------------------------------------*/
void ICU_InstallHandler (ICU_INT_SRC_T int_no, HANDLER_T proc);

#endif  /* STATIC_DISPATCH_TABLE */

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ICU_EnableInt                                                                          */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  int_no  - interrupt number                                                             */
/*                  enable  - TRUE to enable interrupt, FALSE to disable                                   */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine enables/disables interrupt number int_no.                                 */
/*---------------------------------------------------------------------------------------------------------*/
void ICU_EnableInt (ICU_INT_SRC_T int_no, BOOLEAN enable);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ICU_IntEnabled                                                                         */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  int_no    - Number of interrupt to check if enabled                                    */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine checks if given interrupt is enabled                                      */
/*---------------------------------------------------------------------------------------------------------*/
BOOLEAN ICU_IntEnabled(ICU_INT_SRC_T int_no);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ICU_ClearInt                                                                           */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  int_no - interrupt number                                                              */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine clears interrupt number int_no.                                           */
/*---------------------------------------------------------------------------------------------------------*/
void ICU_ClearInt (ICU_INT_SRC_T int_no);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ICU_PendingInt                                                                         */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  int_no - interrupt number                                                              */
/*                                                                                                         */
/* Returns:         interrupts status                                                                      */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine returns TRUE if given int_no interrupt is pending                         */
/*---------------------------------------------------------------------------------------------------------*/
BOOLEAN ICU_PendingInt (ICU_INT_SRC_T int_no);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        null_handler                                                                           */
/*                                                                                                         */
/* Parameters:      none                                                                                   */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine implements a NULL handler (used as dummy or placeholder in dispatch table)*/
/*---------------------------------------------------------------------------------------------------------*/
void null_handler (void);

#endif /* _ICU_IF_H */

