/*---------------------------------------------------------------------------------------------------------*/
/*  Nuvoton Technology Corporation confidential                                                            */
/*                                                                                                         */
/*  Copyright (c) 2009 by Nuvoton Technology Corporation                                                   */
/*  All rights reserved                                                                                    */
/*                                                                                                         */
/*<<<------------------------------------------------------------------------------------------------------*/
/* File Contents:                                                                                          */
/*   itim8_drv.c                                                                                           */
/*            This file contains implementation of Internal 8-bit Timer (ITIM8) module driver              */
/* Project:                                                                                                */
/*            SWC HAL                                                                                      */
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                                INCLUDES                                                 */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
#include "../../../Chips/chip.h"

#include "itim8_drv.h"
#include "itim8_regs.h"


/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                                  TYPES                                                  */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
typedef struct
{
    ITIM8_PERIODIC_EVENT    event;
    void*                   arg;
} ITIM8_EVENT_T;

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                             LOCAL VARIABLES                                             */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
static ITIM8_EVENT_T ITIM8_events[ITIM8_NUM_OF_MODULES];

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                       LOCAL FUNCTIONS DECLARATION                                       */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
static void             ITIM8_SetClockSource_L      (ITIM8_MODULE_T module, ITIM8_SOURCE_CLOCK_T clock);
static DEFS_STATUS      ITIM8_SetRegsByFrequency_L  (ITIM8_MODULE_T module, UINT32 frequency);
static DEFS_STATUS      ITIM8_SetRegsByMicroSec     (ITIM8_MODULE_T module, UINT32 microSec);
static void             ITIM8_Isr                   (UINT16 int_num);

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                              API FUNCTIONS                                              */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_Init                                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                  clock  - Source clock for the ITIM8 module                                             */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine initializes ITIM8 module                                                  */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_Init (
    ITIM8_MODULE_T          module,
    ITIM8_SOURCE_CLOCK_T    clock
)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Stop and reset timer                                                                                */
    /*-----------------------------------------------------------------------------------------------------*/
    (void)ITIM8_StopTimer(module);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set new source clock                                                                                */
    /*-----------------------------------------------------------------------------------------------------*/
    ITIM8_SetClockSource_L(module, clock);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Clear events                                                                                        */
    /*-----------------------------------------------------------------------------------------------------*/
    ITIM8_events[module].event = NULL;
    ITIM8_events[module].arg   = NULL;

    return DEFS_STATUS_OK;
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_StartPeriodicEvent                                                               */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module      - ITIM8 module number                                                      */
/*                  frequency   - Frequency of the periodic event                                          */
/*                  event       - Event callback                                                           */
/*                  eventParam  - Event parameter                                                          */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine starts Timer in periodic event mode                                       */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_StartPeriodicEvent (
    ITIM8_MODULE_T          module,
    UINT32                  frequency,
    ITIM8_PERIODIC_EVENT    event,
    void*                   eventParam
)
{
    DEFS_STATUS ret;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set prescaler and count                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    if ((ret = ITIM8_SetRegsByFrequency_L(module, frequency)) != DEFS_STATUS_OK)
    {
        return ret;
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Updating events array                                                                               */
    /*-----------------------------------------------------------------------------------------------------*/
    ITIM8_events[module].event = event;
    ITIM8_events[module].arg   = eventParam;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Start event only if it is not NULL                                                                  */
    /*-----------------------------------------------------------------------------------------------------*/
    if (event != NULL)
    {
        /*-------------------------------------------------------------------------------------------------*/
        /* Register relevant interrupt                                                                     */
        /*-------------------------------------------------------------------------------------------------*/
        INTERRUPT_REGISTER_AND_ENABLE(ITIM8_INTERRUPT_PROVIDER(module), ITIM8_INTERRUPT(module), ITIM8_Isr,
                                      ITIM8_INTERRUPT_POLARITY(module), ITIM8_INTERRUPT_PRIORITY(module));

        /*-------------------------------------------------------------------------------------------------*/
        /* Enable interrupt                                                                                */
        /*-------------------------------------------------------------------------------------------------*/
        SET_REG_FIELD(ITCTS(module), ITCTS_TO_IE, 1);
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Start the timer                                                                                     */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(ITCTS(module), ITCTS_ITEN, 1);

    return DEFS_STATUS_OK;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_StartPeriodicEventEx                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module      - ITIM8 module number                                                      */
/*                  prescaler   - clock prescaler                                                          */
/*                  counter     - clock counter                                                            */
/*                  event       - Event callback                                                           */
/*                  eventParam  - Event parameter                                                          */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  Extended (or Expert) function. It starts Timer in periodic event mode                  */
/*                  but let you define the prescaler and counter manually.                                 */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_StartPeriodicEventEx (
    ITIM8_MODULE_T          module,
    UINT8                   prescaler,
    UINT16                  counter,
    ITIM8_PERIODIC_EVENT    event,
    void*                   eventParam
)
{

    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set prescaler and count                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    REG_WRITE(ITPRE(module), prescaler);
    REG_WRITE(ITCNT16(module), counter);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Updating events array                                                                               */
    /*-----------------------------------------------------------------------------------------------------*/
    ITIM8_events[module].event = event;
    ITIM8_events[module].arg   = eventParam;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Start event only if it is not NULL                                                                  */
    /*-----------------------------------------------------------------------------------------------------*/
    if (event != NULL)
    {
        /*-------------------------------------------------------------------------------------------------*/
        /* Register relevant interrupt                                                                     */
        /*-------------------------------------------------------------------------------------------------*/
        INTERRUPT_REGISTER_AND_ENABLE(ITIM8_INTERRUPT_PROVIDER(module), ITIM8_INTERRUPT(module), ITIM8_Isr,
                                      ITIM8_INTERRUPT_POLARITY(module), ITIM8_INTERRUPT_PRIORITY(module));

        /*-------------------------------------------------------------------------------------------------*/
        /* Enable interrupt                                                                                */
        /*-------------------------------------------------------------------------------------------------*/
        SET_REG_FIELD(ITCTS(module), ITCTS_TO_IE, 1);
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Start the timer                                                                                     */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(ITCTS(module), ITCTS_ITEN, 1);

    return DEFS_STATUS_OK;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_ReadCounter                                                                      */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM module number.                                                           */
/* Returns:         The current value in the ITIM 16bit couter register                                    */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  Returns the current value in the ITIM 16bit couter register                            */
/*---------------------------------------------------------------------------------------------------------*/
UINT16 ITIM8_ReadCounter(ITIM8_MODULE_T module)
{
    volatile UINT16 t1;

    do 
    {
        t1 = REG_READ(ITCNT16(module)); 
    }
    while (t1 != REG_READ(ITCNT16(module)));

    return t1;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_StopPeriodicEvent                                                                */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine stops periodic event                                                      */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_StopPeriodicEvent (ITIM8_MODULE_T module)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Stop timer                                                                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    (void)ITIM8_StopTimer(module);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Disable relevant interrupt                                                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    INTERRUPT_ENABLE(ITIM8_INTERRUPT_PROVIDER(module), ITIM8_INTERRUPT(module), FALSE);

    return DEFS_STATUS_OK;
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_StartTimer                                                                       */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module      - ITIM8 module number                                                      */
/*                  microSec    - time to count in microsecond                                             */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine starts count down timer with single execution                             */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_StartTimer (
    ITIM8_MODULE_T  module,
    UINT32          microSec
)
{
    DEFS_STATUS ret;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    /*-----------------------------------------------------------------------------------------------------*/
    /* First stop the timer                                                                                */
    /*-----------------------------------------------------------------------------------------------------*/
    (void)ITIM8_StopTimer(module);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set prescaler and count                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    if ((ret = ITIM8_SetRegsByMicroSec(module, microSec)) != DEFS_STATUS_OK)
    {
        return ret;
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Start the timer                                                                                     */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(ITCTS(module), ITCTS_ITEN, 1);

    return DEFS_STATUS_OK;
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_TimerRunning                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine returns true while timer is still running by previous invocation of       */
/*                  ITIM8_StartTimer                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
BOOLEAN ITIM8_TimerRunning (ITIM8_MODULE_T module)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    return (READ_REG_FIELD(ITCTS(module), ITCTS_TO_STS) == 0);
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_StopTimer                                                                        */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine stops the timer started by ITIM8_StartTimer                               */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_StopTimer (ITIM8_MODULE_T module)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Disable interrupt                                                                                   */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(ITCTS(module), ITCTS_TO_IE, 0);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Disable the module                                                                                  */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(ITCTS(module), ITCTS_ITEN, 0);

#ifndef ITIM8_CAPABILITY_DISABLE_WITHOUT_WAIT
    /*-----------------------------------------------------------------------------------------------------*/
    /* Wait till module is disabled (can take several clocks)                                              */
    /*-----------------------------------------------------------------------------------------------------*/
    while (READ_REG_FIELD(ITCTS(module), ITCTS_ITEN) != 0)
    {
        /*-------------------------------------------------------------------------------------------------*/
        /* Disable the module                                                                              */
        /*-------------------------------------------------------------------------------------------------*/
        SET_REG_FIELD(ITCTS(module), ITCTS_ITEN, 0);

        /*-------------------------------------------------------------------------------------------------*/
        /* Small delay loop                                                                                */
        /*-------------------------------------------------------------------------------------------------*/
        DELAY_LOOP(10);
    }
#endif

    return DEFS_STATUS_OK;
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_DelayMicroSec                                                                    */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module      - ITIM8 module number                                                      */
/*                  microSec    - timer to wait in microseconds                                            */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine performs blocking delay for given microseconds                            */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_DelayMicroSec (
    ITIM8_MODULE_T  module,
    UINT32          microSec
)
{
    DEFS_STATUS ret;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Start timer                                                                                         */
    /*-----------------------------------------------------------------------------------------------------*/
    if ((ret = ITIM8_StartTimer(module, microSec)) != DEFS_STATUS_OK)
    {
        return ret;
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Wait till timer count down                                                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    while (ITIM8_TimerRunning(module)) ;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Stop timer                                                                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    (void)ITIM8_StopTimer(module);

    return DEFS_STATUS_OK;
}


#ifdef ITIM8_CAPABILITY_WAKEUP
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_WakeupEnable                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine enables/disables timer wakeup capability                                  */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_WakeupEnable(ITIM8_MODULE_T module, BOOLEAN enable)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Configure wakeup                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(ITCTS(module), ITCTS_TO_WUE, enable);

    return DEFS_STATUS_OK;
}
#endif


/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                             LOCAL FUNCTIONS                                             */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_SetClockSource_L                                                                 */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                  clock  - Clock source                                                                  */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine sets clock source                                                         */
/*---------------------------------------------------------------------------------------------------------*/
static void ITIM8_SetClockSource_L (
    ITIM8_MODULE_T          module,
    ITIM8_SOURCE_CLOCK_T    clock
)
{
    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    /*-----------------------------------------------------------------------------------------------------*/
    /* We must change clock together with disabling the module                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    if (clock == ITIM8_SOURCE_CLOCK_1)
    {
        SET_REG_FIELD(ITCTS(module), ITCTS_CKSEL, 0);
    }
    else if (clock == ITIM8_SOURCE_CLOCK_2)
    {
        SET_REG_FIELD(ITCTS(module), ITCTS_CKSEL, 1);
    }
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_SetRegsByFrequency_L                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module      - ITIM8 module number                                                      */
/*                  frequency   - Frequency of the periodic event                                          */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine configures prescaler and count for a given frequency                      */
/*---------------------------------------------------------------------------------------------------------*/
static DEFS_STATUS ITIM8_SetRegsByFrequency_L (
    ITIM8_MODULE_T  module,
    UINT32          frequency
)
{
    UINT32 clock_freq   = 0;
    UINT32 pre          = 0;
    UINT32 count        = 0;
    UINT32 in_out_ratio = 0;
    UINT8  pre_8b       = 0;
    UINT8  count_8b     = 0;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    if (frequency == 0)
    {
        return DEFS_STATUS_INVALID_PARAMETER;
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Retrive clock frequency                                                                             */
    /*-----------------------------------------------------------------------------------------------------*/
    clock_freq = ITIM8_SOURCE_CLOCK((ITIM8_SOURCE_CLOCK_T)READ_REG_FIELD(ITCTS(module), ITCTS_CKSEL));

    /*-----------------------------------------------------------------------------------------------------*/
    /* Calculate input to output frequency ration                                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    in_out_ratio = DIV_ROUND(clock_freq, frequency);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Check if we can supply such frequency                                                               */
    /*-----------------------------------------------------------------------------------------------------*/
    if (in_out_ratio <= 1)
    {
        /*-------------------------------------------------------------------------------------------------*/
        /* We can't supply frequency higher than (or equal to) the input clock frequency                   */
        /*-------------------------------------------------------------------------------------------------*/
        return DEFS_STATUS_PARAMETER_OUT_OF_RANGE;
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Calculate maximum count (assuming pre == 0)                                                         */
    /*-----------------------------------------------------------------------------------------------------*/
    count =  in_out_ratio - 1;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Check if we can satisfy the frequency with count only                                               */
    /*-----------------------------------------------------------------------------------------------------*/
    if (count > 0xFF)
    {
        /*-------------------------------------------------------------------------------------------------*/
        /* We can NOT satisfy the frequency with count only                                                */
        /* Calculating prescaler                                                                           */
        /*-------------------------------------------------------------------------------------------------*/
        pre = DIV_ROUND(in_out_ratio, 256);

        /*-------------------------------------------------------------------------------------------------*/
        /* Check if we can supply the frequency with combination of PRESCALER and COUNT                    */
        /*-------------------------------------------------------------------------------------------------*/
        if (pre > 0xFF)
        {
            /*---------------------------------------------------------------------------------------------*/
            /* We can't supply the frequency. The wanted frequency is TOO LOW                              */
            /*---------------------------------------------------------------------------------------------*/
            return DEFS_STATUS_PARAMETER_OUT_OF_RANGE;
        }
        /*-------------------------------------------------------------------------------------------------*/
        /* Calculating count                                                                               */
        /*-------------------------------------------------------------------------------------------------*/
        count = DIV_ROUND(clock_freq/(pre+1), frequency) - 1;
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Write calculated values to registers                                                                */
    /*-----------------------------------------------------------------------------------------------------*/
    pre_8b      = (UINT8)(pre   & 0xFF);
    count_8b    = (UINT8)(count & 0xFF);

    REG_WRITE(ITPRE(module), pre_8b);
    REG_WRITE(ITCNT(module), count_8b);

    return DEFS_STATUS_OK;
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_SetRegsByMicroSec                                                                */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module      - ITIM8 module number                                                      */
/*                  microSec    - Timers time for counting                                                 */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine set prescaler and count registers by micro-seconds                        */
/*---------------------------------------------------------------------------------------------------------*/
static DEFS_STATUS ITIM8_SetRegsByMicroSec (
    ITIM8_MODULE_T  module,
    UINT32          microSec
)
{
    UINT32 freq;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Parameters check                                                                                    */
    /*-----------------------------------------------------------------------------------------------------*/
    ASSERT (module < ITIM8_NUM_OF_MODULES);

    if (microSec == 0)
    {
        return DEFS_STATUS_INVALID_PARAMETER;
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* Convert micro-seconds to frequency                                                                  */
    /*-----------------------------------------------------------------------------------------------------*/
    freq = DIV_ROUND(1000000UL, microSec);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Set registers by frequency                                                                          */
    /*-----------------------------------------------------------------------------------------------------*/
    return ITIM8_SetRegsByFrequency_L(module, freq);
}



/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_Isr                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  int_num - number of triggered interrupt                                                */
/*                                                                                                         */
/* Returns:         none                                                                                   */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  ITIM8 Interrupt service routine                                                        */
/*---------------------------------------------------------------------------------------------------------*/
static void ITIM8_Isr (UINT16 int_num)
{
    UINT    module;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Find module number from interrupt number                                                            */
    /*-----------------------------------------------------------------------------------------------------*/
    for (module = 0; module < ITIM8_NUM_OF_MODULES; ++module)
    {
        if (ITIM8_INTERRUPT(module) == int_num)
        {
            break;
        }
    }

    /*-----------------------------------------------------------------------------------------------------*/
    /* No module number found, exit                                                                        */
    /*-----------------------------------------------------------------------------------------------------*/
    if (module == ITIM8_NUM_OF_MODULES)
        return;

    /*-----------------------------------------------------------------------------------------------------*/
    /* Clear ITCTS_TO_STS                                                                                  */
    /*-----------------------------------------------------------------------------------------------------*/
    SET_REG_FIELD(ITCTS(module), ITCTS_TO_STS, 1);

    /*-----------------------------------------------------------------------------------------------------*/
    /* Handle pending interrupt                                                                            */
    /*-----------------------------------------------------------------------------------------------------*/
    EXECUTE_FUNC(ITIM8_events[module].event, (ITIM8_events[module].arg));
}

