/*---------------------------------------------------------------------------------------------------------*/
/*  Nuvoton Technology Corporation confidential                                                            */
/*                                                                                                         */
/*  Copyright (c) 2009 by Nuvoton Technology Corporation                                                   */
/*  All rights reserved                                                                                    */
/*                                                                                                         */
/*<<<------------------------------------------------------------------------------------------------------*/
/* File Contents:                                                                                          */
/*   itim8_if.h                                                                                            */
/*            This file contains interface for Internal 8-bit Timer (ITIM8) module                         */
/* Project:                                                                                                */
/*            SWC HAL                                                                                      */
/*---------------------------------------------------------------------------------------------------------*/

#ifndef __ITIM8_IF_H__
#define __ITIM8_IF_H__

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                                 INCLUDES                                                */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
#include "../../Chips/chip.h"

#if defined ITIM8_MODULE_TYPE
#include __MODULE_HEADER(itim8, ITIM8_MODULE_TYPE)
#endif

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                           TYPES & DEFINITIONS                                           */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* ITIM8 modules enum                                                                                      */
/*---------------------------------------------------------------------------------------------------------*/
typedef enum
{
    ITIM8_MODULE_1      = 0,
    ITIM8_MODULE_2      = 1,
    ITIM8_MODULE_3      = 2
} ITIM8_MODULE_T;

/*---------------------------------------------------------------------------------------------------------*/
/* ITIM8 clock sources                                                                                     */
/*---------------------------------------------------------------------------------------------------------*/
typedef enum
{
    ITIM8_SOURCE_CLOCK_1 = 0,
    ITIM8_SOURCE_CLOCK_2 = 1,

    ITIM8_SOURCE_CLOCK_UNDEF= 0xFF

} ITIM8_SOURCE_CLOCK_T;

/*---------------------------------------------------------------------------------------------------------*/
/* ITIM8 periodic event callback                                                                           */
/*---------------------------------------------------------------------------------------------------------*/
typedef void (*ITIM8_PERIODIC_EVENT)(void* arg);

/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/
/*                                                 API                                                     */
/*---------------------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_Init                                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                  clock  - Source clock for the ITIM8 module                                             */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine initializes ITIM8 module                                                  */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_Init                  (ITIM8_MODULE_T module, ITIM8_SOURCE_CLOCK_T clock);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_StartPeriodicEvent                                                               */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module      - ITIM8 module number                                                      */
/*                  frequency   - Frequency of the periodic event                                          */
/*                  event       - Event callback                                                           */
/*                  eventParam  - Event parameter                                                          */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine starts Timer in periodic event mode                                       */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_StartPeriodicEvent    (ITIM8_MODULE_T module, UINT32 frequency, ITIM8_PERIODIC_EVENT event, void* eventParam);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_StartPeriodicEventEx                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module      - ITIM8 module number                                                      */
/*                  prescaler   - clock prescaler                                                          */
/*                  counter     - clock counter                                                            */
/*                  event       - Event callback                                                           */
/*                  eventParam  - Event parameter                                                          */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  Extended (or Expert) function. It starts Timer in periodic event mode                  */
/*                  but let you define the prescaler and counter manually.                                 */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_StartPeriodicEventEx (ITIM8_MODULE_T module, UINT8 prescaler, UINT16 ounter, ITIM8_PERIODIC_EVENT event, void* eventParam);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_ReadCounter                                                                      */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM module number.                                                           */
/* Returns:         The current value in the ITIM 16bit couter register                                    */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  Returns the current value in the ITIM 16bit couter register                            */
/*---------------------------------------------------------------------------------------------------------*/
UINT16 ITIM8_ReadCounter(ITIM8_MODULE_T module);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_StopPeriodicEvent                                                                */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine stops periodic event                                                      */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_StopPeriodicEvent     (ITIM8_MODULE_T module);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_StartTimer                                                                       */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module      - ITIM8 module number                                                      */
/*                  microSec    - time to count in microsecond                                             */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine starts count down timer with single execution                             */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_StartTimer            (ITIM8_MODULE_T module, UINT32 microSec);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_TimerRunning                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine returns true while timer is still running by previous invocation of       */
/*                  ITIM8_StartTimer                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
BOOLEAN     ITIM8_TimerRunning          (ITIM8_MODULE_T module);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_StopTimer                                                                        */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine stops the timer started by ITIM8_StartTimer                               */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_StopTimer             (ITIM8_MODULE_T module);

/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_DelayMicroSec                                                                    */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module      - ITIM8 module number                                                      */
/*                  microSec    - timer to wait in microseconds                                            */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine performs blocking delay for given microseconds                            */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_DelayMicroSec         (ITIM8_MODULE_T module, UINT32 microSec);


#ifdef ITIM8_CAPABILITY_WAKEUP
/*---------------------------------------------------------------------------------------------------------*/
/* Function:        ITIM8_WakeupEnable                                                                     */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*                  module - ITIM8 module number                                                           */
/*                                                                                                         */
/* Returns:                                                                                                */
/* Side effects:                                                                                           */
/* Description:                                                                                            */
/*                  This routine enables/disables timer wakeup capability                                  */
/*---------------------------------------------------------------------------------------------------------*/
DEFS_STATUS ITIM8_WakeupEnable(ITIM8_MODULE_T module, BOOLEAN enable);
#endif

#endif  /* __ITIM8_IF_H__ */

